@@ -66,7 +66,7 @@ const char *strTemplate[] = {
 
 
 GrepTool::GrepTool(QWidget *parent, const char *name)
-  : QWidget(parent, name/*, false*/), childproc(0)
+  : QWidget(parent, name/*, false*/), m_fixFocus(true), childproc(0)
 {
   setCaption(i18n("Find in Files"));
   config = KGlobal::config();
@@ -527,4 +527,19 @@ bool GrepTool::eventFilter( QObject *o, QEvent *e )
   return QWidget::eventFilter( o, e );
 }
 
+void GrepTool::focusInEvent ( QFocusEvent * ev )
+{
+  QWidget::focusInEvent(ev);
+  if (m_fixFocus) {
+    m_fixFocus = false;
+    cmbPattern->setFocus();
+  }
+}
+
+void GrepTool::showEvent( QShowEvent * ev )
+{
+  QWidget::showEvent(ev);
+  m_fixFocus = true;
+}
+
 #include "kategrepdialog.moc"
