@@ -288,8 +288,9 @@ void BGDialog::makeReadOnly()
     m_buttonGetNew->setEnabled( false );
 }
 
-void BGDialog::load()
+void BGDialog::load( bool useDefaults )
 {
+   m_pGlobals->getConfig()->setReadDefaults( useDefaults );
    m_pGlobals->readSettings();
    m_eDesk = m_pGlobals->commonDeskBackground() ? 0 : m_desk;
    getEScreen();
@@ -321,7 +322,7 @@ void BGDialog::load()
       m_wallpaperPos = KBackgroundSettings::Centred; // Default
 
    updateUI();
-   emit changed(false);
+   emit changed(useDefaults);
 }
 
 void BGDialog::save()
@@ -358,51 +359,7 @@ void BGDialog::save()
 
 void BGDialog::defaults()
 {
-   m_pGlobals->setCommonScreenBackground(_defCommonScreen);
-   m_pGlobals->setCommonDeskBackground(_defCommonDesk);
-   m_pGlobals->setLimitCache(_defLimitCache);
-   m_pGlobals->setCacheSize(_defCacheSize);
-   m_comboWallpaperPos->setCurrentItem( 0 );
-   m_eDesk = _defCommonDesk ? 0 : m_desk;
-   getEScreen();
-   
-   for ( unsigned i = 0; i < m_numDesks; ++i )
-      m_pGlobals->setDrawBackgroundPerScreen( i, false );
-
-   KBackgroundRenderer *r = eRenderer();
-
-   if (r->isActive())
-      r->stop();
-
-   if (QPixmap::defaultDepth() > 8)
-   {
-      r->setBackgroundMode(_defBackgroundMode);
-   }
-   else
-   {
-      r->setBackgroundMode(KBackgroundSettings::Flat);
-   }
-
-   r->setColorA(_defColorA);
-   r->setColorB(_defColorB);
-   r->setWallpaperMode(_defWallpaperMode);
-   r->setMultiWallpaperMode(_defMultiMode);
-
-   m_slideShowRandom = _defMultiMode;
-   if (m_slideShowRandom == KBackgroundSettings::NoMultiRandom)
-      m_slideShowRandom = KBackgroundSettings::Random;
-   if (m_slideShowRandom == KBackgroundSettings::NoMulti)
-      m_slideShowRandom = KBackgroundSettings::InOrder;
-
-   r->setBlendMode(_defBlendMode);
-   r->setBlendBalance(_defBlendBalance);
-   r->setReverseBlending(_defReverseBlending);
-
-   updateUI();
-
-   m_copyAllDesktops = true;
-   m_copyAllScreens = true;
-   emit changed(true);
+	load( true );
 }
 
 QString BGDialog::quickHelp() const
@@ -537,8 +494,6 @@ void BGDialog::loadWallpaperFilesList() {
          imageCaption = KStringHandler::capwords(imageCaption);
       }
 
-      imageCaption = KStringHandler::rEmSqueeze(imageCaption, m_urlWallpaperBox->fontMetrics(), 11);
-
       // avoid name collisions
       QString rs = imageCaption;
       QString lrs = rs.lower();
@@ -587,8 +542,6 @@ void BGDialog::loadWallpaperFilesList() {
             imageCaption = KStringHandler::capwords(imageCaption);
          }
 
-         imageCaption = KStringHandler::rEmSqueeze(imageCaption, m_urlWallpaperBox->fontMetrics(), 11);
-
          // avoid name collisions
          QString rs = imageCaption;
          QString lrs = rs.lower();
@@ -637,7 +590,7 @@ void BGDialog::setWallpaper(const QString &s)
          i--;
          comboWallpaper->removeItem(i);
       }
-      comboWallpaper->insertItem(KStringHandler::rEmSqueeze(imageCaption, m_urlWallpaperBox->fontMetrics(), 11));
+      comboWallpaper->insertItem(imageCaption);
       m_wallpaper[s] = i;
       comboWallpaper->setCurrentItem(i);
    }
@@ -835,14 +788,16 @@ void BGDialog::slotPreviewDone(int desk_done, int screen_done)
 
    KBackgroundRenderer * r = m_renderer[m_eDesk][(m_eScreen>1) ? (screen_done+2) : m_eScreen];
 
-   if (!r->image())
+   if (r->image().isNull())
       return;
 
+   r->saveCacheFile();
+
    KPixmap pm;
    if (QPixmap::defaultDepth() < 15)
-      pm.convertFromImage(*r->image(), KPixmap::LowColor);
+      pm.convertFromImage(r->image(), KPixmap::LowColor);
    else
-      pm.convertFromImage(*r->image());
+      pm.convertFromImage(r->image());
 
    if ( m_eScreen == 0 )
    {
@@ -1218,6 +1173,13 @@ void BGDialog::slotAdvanced()
         m_previewUpdates = true;
         return;
     }
+
+    r->setBackgroundMode(dlg.backgroundMode());
+    if (dlg.backgroundMode() == KBackgroundSettings::Program)
+    {
+        r->setProgram(dlg.backgroundProgram());
+    }
+
     int cacheSize = dlg.cacheSize();
     if (cacheSize)
     {
