@@ -303,8 +303,13 @@ void KColorScheme::setColorName( const QString &name, int id )
 }
 
 void KColorScheme::load()
+{
+   load( false );
+}
+void KColorScheme::load( bool useDefaults )
 {
     KConfig *config = KGlobal::config();
+    config->setReadDefaults(  useDefaults );
     config->setGroup("KDE");
     sCurrentScheme = config->readEntry("colorScheme");
 
@@ -324,7 +329,7 @@ void KColorScheme::load()
     bool exportColors = cfg.readBoolEntry("exportKDEColors", true);
     cbExportColors->setChecked(exportColors);
 
-    emit changed(false);
+    emit changed( useDefaults );
 }
 
 
@@ -419,20 +424,7 @@ void KColorScheme::save()
 
 void KColorScheme::defaults()
 {
-    readScheme(1);
-    sList->setCurrentItem(1);
-
-    cbShadeList->setChecked(cs->shadeSortColumn);
-
-    cs->drawSampleWidgets();
-    slotWidgetColor(wcCombo->currentItem());
-    sb->blockSignals(true);
-    sb->setValue(cs->contrast);
-    sb->blockSignals(false);
-
-    cbExportColors->setChecked(true);
-
-    emit changed(true);
+   load( true );
 }
 
 void KColorScheme::sliderValueChanged( int val )
@@ -545,7 +537,7 @@ void KColorScheme::slotAdd()
             if (sName == sList->text(i))
             {
                 exists = i;
-                int result = KMessageBox::warningContinueCancel( 0,
+                int result = KMessageBox::warningContinueCancel( this,
                    i18n("A color scheme with the name '%1' already exists.\n"
                         "Do you want to overwrite it?\n").arg(sName),
                    i18n("Save Color Scheme"),
