@@ -14,7 +14,7 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 */
 
@@ -26,7 +26,7 @@
 
 
 class QSplitter;
-class QTabWidget;
+class QWidgetStack;
 
 class KToggleAction;
 class KAction;
@@ -63,32 +63,34 @@ protected slots:
   void activateSmallIcons();
   void activateMediumIcons();
   void activateLargeIcons();
+  void activateHugeIcons();
 
   void deleteDummyAbout();
 
+  void slotSearchChanged(const QString &);
+  void slotHandbookRequest();
   void slotHelpRequest();
 
   void changedModule(ConfigModule *changed);
 
   bool queryClose();
-    void slotConfigureKeys();
 
 private:
 
   QString handleAmpersand( QString ) const;
 
   QSplitter      *_splitter;
-  QTabWidget     *_tab;
+  QWidgetStack   *_stack;
   DockContainer  *_dock;
   ModuleTitle    *_title;
 
   KToggleAction *tree_view, *icon_view;
-  KToggleAction *icon_small, *icon_medium, *icon_large;
+  KToggleAction *icon_small, *icon_medium, *icon_large, *icon_huge;
   KAction *report_bug, *about_module;
 
-  IndexWidget  *_indextab;
-  SearchWidget *_searchtab;
-  HelpWidget   *_helptab;
+  IndexWidget  *_index;
+  SearchWidget *_search;
+  HelpWidget   *_help;
 
   ConfigModule     *_active;
   ConfigModuleList *_modules;
