@@ -16,30 +16,24 @@
  */
 
 #include <qcheckbox.h>
-#include <qslider.h>
-#include <qwhatsthis.h>
-#include <qlineedit.h>
-#include <qlistview.h>
-#include <qbuttongroup.h>
-#include <qradiobutton.h>
-#include <qvalidator.h>
-#include <qtoolbutton.h>
 #include <qdir.h>
-#include <qtooltip.h>
+#include <qlabel.h>
 #include <qlayout.h>
+#include <qpushbutton.h>
+#include <qradiobutton.h>
 
-#include <kconfig.h>
-#include <kglobal.h>
-#include <knuminput.h>
-#include <klocale.h>
-#include <klistview.h>
-#include <kmessagebox.h>
-#include <kdesktopfile.h>
-#include <kstandarddirs.h>
-#include <kiconloader.h>
 #include <kapplication.h>
+#include <kdebug.h>
+#include <kdesktopfile.h>
+#include <kiconloader.h>
+#include <klistview.h>
+#include <klocale.h>
+#include <kmessagebox.h>
+#include <knuminput.h>
+#include <kstandarddirs.h>
 
 #include "main.h"
+
 #include "menutab_impl.h"
 #include "menutab_impl.moc"
 
@@ -71,15 +65,7 @@ MenuTab::MenuTab( QWidget *parent, const char* name )
     m_quickBrowserMenu(0)
 {
     // connections
-    connect(m_formatSimple, SIGNAL(clicked()), SIGNAL(changed()));
-    connect(m_formatNameDesc, SIGNAL(clicked()), SIGNAL(changed()));
-    connect(m_formDescName, SIGNAL(clicked()), SIGNAL(changed()));
-    connect(m_showPixmap, SIGNAL(clicked()), SIGNAL(changed()));
-    connect(m_hiddenFiles, SIGNAL(clicked()), SIGNAL(changed()));
-    connect(m_maxQuickBrowserItems, SIGNAL(valueChanged(int)), SIGNAL(changed()));
-    connect(m_showRecent, SIGNAL(clicked()), SIGNAL(changed()));
-    connect(m_showFrequent, SIGNAL(clicked()), SIGNAL(changed()));
-    connect(m_maxQuickStartItems, SIGNAL(valueChanged(int)), SIGNAL(changed()));
+    connect(m_editKMenuButton, SIGNAL(clicked()), SLOT(launchMenuEditor()));
 
     m_browserGroupLayout->setColStretch( 1, 1 );
     m_pRecentOrderGroupLayout->setColStretch( 1, 1 );
@@ -87,31 +73,16 @@ MenuTab::MenuTab( QWidget *parent, const char* name )
 
 void MenuTab::load()
 {
-    KConfig c(KickerConfig::configName(), false, false);
-    c.setGroup("KMenu");
+   load( false );
+}
 
-    m_showPixmap->setChecked(c.readBoolEntry("UseSidePixmap", true));
+void MenuTab::load( bool useDefaults )
+{
+    KSharedConfig::Ptr c = KSharedConfig::openConfig(KickerConfig::the()->configName());
+    
+    c->setReadDefaults( useDefaults );
 
-    c.setGroup("menus");
-
-    m_hiddenFiles->setChecked(c.readBoolEntry("ShowHiddenFiles", false));
-    m_maxQuickBrowserItems->setValue(c.readNumEntry("MaxEntries2", 30));
-
-    if (c.readBoolEntry("DetailedMenuEntries", true))
-    {
-        if (c.readBoolEntry("DetailedEntriesNamesFirst", true))
-        {
-            m_formatNameDesc->setChecked(true);
-        }
-        else
-        {
-            m_formDescName->setChecked(true);
-        }
-    }
-    else
-    {
-        m_formatSimple->setChecked(true);
-    }
+    c->setGroup("menus");
 
     m_subMenus->clear();
 
@@ -120,7 +91,7 @@ void MenuTab::load()
                                       i18n("Bookmarks"),
                                       QString::null,
                                       SmallIcon("bookmark"),
-                                      c.readBoolEntry("UseBookmarks", true));
+                                      c->readBoolEntry("UseBookmarks", false));
     connect(m_bookmarkMenu, SIGNAL(toggled(bool)), SIGNAL(changed()));
 
     // show the quick menus menu?
@@ -128,10 +99,12 @@ void MenuTab::load()
                                           i18n("Quick Browser"),
                                           QString::null,
                                           SmallIcon("kdisknav"),
-                                          c.readBoolEntry("UseBrowser", true));
+                                          c->readBoolEntry("UseBrowser", false));
     connect(m_quickBrowserMenu, SIGNAL(toggled(bool)), SIGNAL(changed()));
 
-    QStringList ext = c.readListEntry("Extensions");
+    QStringList ext_default;
+    ext_default << "prefmenu.desktop" << "systemmenu.desktop";
+    QStringList ext = c->readListEntry("Extensions", ext_default);
     QStringList dirs = KGlobal::dirs()->findDirs("data", "kicker/menuext");
     kSubMenuItem* menuItem(0);
     for (QStringList::ConstIterator dit=dirs.begin(); dit!=dirs.end(); ++dit)
@@ -150,31 +123,17 @@ void MenuTab::load()
         }
     }
 
-    if (c.readBoolEntry("RecentVsOften", false))
-        m_showRecent->setChecked(true);
-    else
-        m_showFrequent->setChecked(true);
+    m_showFrequent->setChecked(true);
 
-    m_maxQuickStartItems->setValue(c.readNumEntry("NumVisibleEntries", 5));
+    if ( useDefaults )
+       emit changed();
 }
 
 void MenuTab::save()
 {
-    KConfig c(KickerConfig::configName(), false, false);
+    KSharedConfig::Ptr c = KSharedConfig::openConfig(KickerConfig::the()->configName());
 
-    c.setGroup("KMenu");
-
-    c.writeEntry("UseSidePixmap", m_showPixmap->isChecked());
-
-    c.setGroup("menus");
-
-    c.writeEntry("MaxEntries2", m_maxQuickBrowserItems->value());
-
-    c.writeEntry("DetailedMenuEntries", !m_formatSimple->isChecked());
-    c.writeEntry("DetailedEntriesNamesFirst", m_formatNameDesc->isChecked());
-    c.writeEntry("ShowHiddenFiles", m_hiddenFiles->isChecked());
-    c.writeEntry("NumVisibleEntries", m_maxQuickStartItems->value());
-    c.writeEntry("RecentVsOften", m_showRecent->isChecked());
+    c->setGroup("menus");
 
     QStringList ext;
     QListViewItem *item(0);
@@ -183,34 +142,25 @@ void MenuTab::save()
         bool isOn = static_cast<kSubMenuItem*>(item)->isOn();
         if (item == m_bookmarkMenu)
         {
-            c.writeEntry("UseBookmarks", isOn);
+            c->writeEntry("UseBookmarks", isOn);
         }
         else if (item == m_quickBrowserMenu)
         {
-            c.writeEntry("UseBrowser", isOn);
+            c->writeEntry("UseBrowser", isOn);
         }
         else if (isOn)
         {
             ext << static_cast<kSubMenuItem*>(item)->desktopFile();
         }
     }
-    c.writeEntry("Extensions", ext);
+    c->writeEntry("Extensions", ext);
 
-    c.sync();
+    c->sync();
 }
 
 void MenuTab::defaults()
 {
-  m_showPixmap->setChecked(true);
-  m_maxQuickBrowserItems->setValue(30);
-  m_formatNameDesc->setChecked(true);
-  m_showRecent->setChecked(true);
-  m_hiddenFiles->setChecked(false);
-  m_bookmarkMenu->setOn(true);
-  m_quickBrowserMenu->setOn(true);
-
-  m_showFrequent->setChecked(true);
-  m_maxQuickStartItems->setValue(5);
+   load( true );
 }
 
 void MenuTab::launchMenuEditor()
@@ -229,4 +179,3 @@ void MenuTab::launchMenuEditor()
                            i18n("Application Missing"));
     }
 }
-
