@@ -217,14 +217,26 @@ NICList* findNICs()
             tmp->netmask=i18n("Unknown");
 
          ifcopy=*ifr;
+         result=-1;  // if none of the two #ifs below matches, ensure that result!=0 so that "Unknown" is returned as result
+#ifdef SIOCGIFHWADDR
          result=ioctl(sockfd,SIOCGIFHWADDR,&ifcopy);
          if (result==0)
          {
             char *n = &ifcopy.ifr_ifru.ifru_hwaddr.sa_data[0];
             tmp->HWaddr = HWaddr2String(n);
          }
-         else
+#elif defined SIOCGENADDR
+         result=ioctl(sockfd,SIOCGENADDR,&ifcopy);
+         if (result==0)
+         {
+            char *n = &ifcopy.ifr_ifru.ifru_enaddr[0];
+            tmp->HWaddr = HWaddr2String(n);
+         }
+#endif
+         if (result!=0)
+         {
             tmp->HWaddr = i18n("Unknown");
+         }
 
          nl->append(tmp);
          break;
