@@ -82,7 +82,8 @@ const QStringList& TaskbarConfig::actionList()
             << I18N_NOOP("Activate, Raise or Minimize Task")
             << I18N_NOOP("Activate Task") << I18N_NOOP("Raise Task")
             << I18N_NOOP("Lower Task") << I18N_NOOP("Minimize Task")
-            << I18N_NOOP("To Current Desktop") );
+            << I18N_NOOP("To Current Desktop")
+            << I18N_NOOP("Close Task") );
     return list;
 }
 
@@ -166,6 +167,7 @@ TaskbarConfig::TaskbarConfig(QWidget *parent, const char* name, const QStringLis
     {
         m_widget->showAllScreens->hide();
     }
+    connect( m_widget->showAllScreens, SIGNAL( stateChanged( int )), SLOT( changed()));
 
     KAboutData *about = new KAboutData(I18N_NOOP("kcmtaskbar"),
                                        I18N_NOOP("KDE Taskbar Control Module"),
