@@ -66,10 +66,7 @@
 
 KRootWm * KRootWm::s_rootWm = 0;
 
-
-// for multihead
-extern int kdesktop_screen_number;
-
+extern QCString kdesktop_name, kicker_name, kwin_name;
 
 KRootWm::KRootWm(KDesktop* _desktop) : QObject(_desktop)
 {
@@ -556,15 +553,9 @@ void KRootWm::activateMenu( menuChoice choice, const QPoint& global )
       // This allows the menu to disappear when clicking on the background another time
       XUngrabPointer(qt_xdisplay(), CurrentTime);
       XSync(qt_xdisplay(), False);
-      // Ask kicker to showup the menu
-      // make sure we send the message to the correct kicker
-      QCString appname;
-      if (kdesktop_screen_number == 0)
-	  appname = "kicker";
-      else
-	  appname.sprintf("kicker-screen-%d", kdesktop_screen_number);
 
-      DCOPRef( appname.data(), appname.data() ).send( "popupKMenu", global );
+      // Ask kicker to showup the menu
+      DCOPRef( kicker_name, kicker_name ).send( "popupKMenu", global );
       break;
     }
     case CUSTOMMENU1:
@@ -747,7 +738,7 @@ void KRootWm::slotConfigureDesktop() {
 
 void KRootWm::slotConfigClosed()
 {
-    delete m_configDialog;
+    m_configDialog->delayedDestruct();
     m_configDialog = 0;
 }
 
@@ -756,53 +747,28 @@ void KRootWm::slotToggleDesktopMenu()
     KDesktopSettings::setShowMenubar( !(m_bShowMenuBar && menuBar) );
     KDesktopSettings::writeConfig();
 
-    // make sure we send the message to the correct screen
-    QCString appname;
-    if (kdesktop_screen_number == 0)
-	appname = "kdesktop";
-    else
-	appname.sprintf("kdesktop-screen-%d", kdesktop_screen_number);
-
     QByteArray data;
-    kapp->dcopClient()->send( appname.data(), "KDesktopIface", "configure()", data);
+    kapp->dcopClient()->send( kdesktop_name, "KDesktopIface", "configure()", data);
     // for the standalone menubar setting
     kapp->dcopClient()->send( "menuapplet*", "menuapplet", "configure()", data );
-    kapp->dcopClient()->send( "kicker", "kicker", "configureMenubar()", data );
+    kapp->dcopClient()->send( kicker_name, kicker_name, "configureMenubar()", data );
     kapp->dcopClient()->send( "kwin*", "", "reconfigure()", data );
 }
 
 
 void KRootWm::slotUnclutterWindows()
 {
-    QCString appname;
-    if (kdesktop_screen_number == 0)
-	appname = "kwin";
-    else
-	appname.sprintf("kwin-screen-%d", kdesktop_screen_number);
-
-    kapp->dcopClient()->send(appname.data(), "KWinInterface", "unclutterDesktop()", "");
+    kapp->dcopClient()->send(kwin_name, "KWinInterface", "unclutterDesktop()", "");
 }
 
 
 void KRootWm::slotCascadeWindows() {
-    QCString appname;
-    if (kdesktop_screen_number == 0)
-	appname = "kwin";
-    else
-	appname.sprintf("kwin-screen-%d", kdesktop_screen_number);
-
-    kapp->dcopClient()->send(appname.data(), "KWinInterface", "cascadeDesktop()", "");
+    kapp->dcopClient()->send(kwin_name, "KWinInterface", "cascadeDesktop()", "");
 }
 
 
 void KRootWm::slotLock() {
-    QCString appname;
-    if (kdesktop_screen_number == 0)
-	appname = "kdesktop";
-    else
-	appname.sprintf("kdesktop-screen-%d", kdesktop_screen_number);
-
-    kapp->dcopClient()->send(appname, "KScreensaverIface", "lock()", "");
+    kapp->dcopClient()->send(kdesktop_name, "KScreensaverIface", "lock()", "");
 }
 
 
