@@ -170,7 +170,9 @@ LockProcess::LockProcess(bool child, bool useBlankOnly)
         if (on)
         {
             connect(&mCheckDPMS, SIGNAL(timeout()), SLOT(checkDPMSActive()));
-            mCheckDPMS.start(60000);
+            // we can save CPU if we stop it as quickly as possible
+            // but we waste CPU if we check too often -> so take 10s
+            mCheckDPMS.start(10000);
         }
     }
 #endif
@@ -191,17 +193,23 @@ LockProcess::~LockProcess()
     }
 }
 
-static int sigterm_pipe[2];
+static int signal_pipe[2];
 
 static void sigterm_handler(int)
 {
-    char tmp;
-    ::write( sigterm_pipe[1], &tmp, 1);
+    char tmp = 'T';
+    ::write( signal_pipe[1], &tmp, 1);
+}
+
+static void sighup_handler(int)
+{
+    char tmp = 'H';
+    ::write( signal_pipe[1], &tmp, 1);
 }
 
 void LockProcess::timerEvent(QTimerEvent *ev)
 {
-	if (ev->timerId() == mAutoLogoutTimerId)
+	if (mAutoLogout && ev->timerId() == mAutoLogoutTimerId)
 	{
 		killTimer(mAutoLogoutTimerId);
 		AutoLogout autologout(this);
@@ -211,13 +219,7 @@ void LockProcess::timerEvent(QTimerEvent *ev)
 
 void LockProcess::setupSignals()
 {
-    // ignore SIGHUP
     struct sigaction act;
-    act.sa_handler=SIG_IGN;
-    sigemptyset(&(act.sa_mask));
-    sigaddset(&(act.sa_mask), SIGHUP);
-    act.sa_flags = 0;
-    sigaction(SIGHUP, &act, 0L);
     // ignore SIGINT
     act.sa_handler=SIG_IGN;
     sigemptyset(&(act.sa_mask));
@@ -236,17 +238,30 @@ void LockProcess::setupSignals()
     sigaddset(&(act.sa_mask), SIGTERM);
     act.sa_flags = 0;
     sigaction(SIGTERM, &act, 0L);
+    // SIGHUP forces lock
+    act.sa_handler= sighup_handler;
+    sigemptyset(&(act.sa_mask));
+    sigaddset(&(act.sa_mask), SIGHUP);
+    act.sa_flags = 0;
+    sigaction(SIGHUP, &act, 0L);
 
-    pipe(sigterm_pipe);
-    QSocketNotifier* notif = new QSocketNotifier(sigterm_pipe[0],
+    pipe(signal_pipe);
+    QSocketNotifier* notif = new QSocketNotifier(signal_pipe[0],
 	QSocketNotifier::Read, this );
-    connect( notif, SIGNAL(activated(int)), SLOT(sigtermPipeSignal()));
+    connect( notif, SIGNAL(activated(int)), SLOT(signalPipeSignal()));
 }
 
 
-void LockProcess::sigtermPipeSignal()
+void LockProcess::signalPipeSignal()
 {
-    quitSaver();
+    char tmp;
+    ::read( signal_pipe[0], &tmp, 1);
+    if( tmp == 'T' )
+        quitSaver();
+    else if( tmp == 'H' ) {
+        if( !mLocked )
+            startLock();
+    }
 }
 
 //---------------------------------------------------------------------------
@@ -406,17 +421,43 @@ void LockProcess::createSaverWindow()
 #ifdef HAVE_GLXCHOOSEVISUAL
     if( mOpenGLVisual )
     {
-        int attribs[] = { GLX_RGBA, GLX_DOUBLEBUFFER, GLX_DEPTH_SIZE, x11Depth(), None };
-        if( XVisualInfo* i = glXChooseVisual( x11Display(), x11Screen(), attribs ))
+        static int attribs[][ 15 ] =
         {
-            visual = i->visual;
-            static Colormap colormap = 0;
-            if( colormap != 0 )
-                XFreeColormap( x11Display(), colormap );
-            colormap = XCreateColormap( x11Display(), RootWindow( x11Display(), x11Screen()), visual, AllocNone );
-            attrs.colormap = colormap;
-            flags |= CWColormap;
-            XFree( i );
+        #define R GLX_RED_SIZE
+        #define G GLX_GREEN_SIZE
+        #define B GLX_BLUE_SIZE
+            { GLX_RGBA, R, 8, G, 8, B, 8, GLX_DEPTH_SIZE, 8, GLX_DOUBLEBUFFER, GLX_STENCIL_SIZE, 1, None },
+            { GLX_RGBA, R, 4, G, 4, B, 4, GLX_DEPTH_SIZE, 4, GLX_DOUBLEBUFFER, GLX_STENCIL_SIZE, 1, None },
+            { GLX_RGBA, R, 8, G, 8, B, 8, GLX_DEPTH_SIZE, 8, GLX_DOUBLEBUFFER, None },
+            { GLX_RGBA, R, 4, G, 4, B, 4, GLX_DEPTH_SIZE, 4, GLX_DOUBLEBUFFER, None },
+            { GLX_RGBA, R, 8, G, 8, B, 8, GLX_DEPTH_SIZE, 8, GLX_STENCIL_SIZE, 1, None },
+            { GLX_RGBA, R, 4, G, 4, B, 4, GLX_DEPTH_SIZE, 4, GLX_STENCIL_SIZE, 1, None },
+            { GLX_RGBA, R, 8, G, 8, B, 8, GLX_DEPTH_SIZE, 8, None },
+            { GLX_RGBA, R, 4, G, 4, B, 4, GLX_DEPTH_SIZE, 4, None },
+            { GLX_RGBA, GLX_DEPTH_SIZE, 8, GLX_DOUBLEBUFFER, GLX_STENCIL_SIZE, 1, None },
+            { GLX_RGBA, GLX_DEPTH_SIZE, 8, GLX_DOUBLEBUFFER, None },
+            { GLX_RGBA, GLX_DEPTH_SIZE, 8, GLX_STENCIL_SIZE, 1, None },
+            { GLX_RGBA, GLX_DEPTH_SIZE, 8, None }
+        #undef R
+        #undef G
+        #undef B
+        };
+        for( unsigned int i = 0;
+             i < sizeof( attribs ) / sizeof( attribs[ 0 ] );
+             ++i )
+        {
+            if( XVisualInfo* info = glXChooseVisual( x11Display(), x11Screen(), attribs[ i ] ))
+            {
+                visual = info->visual;
+                static Colormap colormap = 0;
+                if( colormap != 0 )
+                    XFreeColormap( x11Display(), colormap );
+                colormap = XCreateColormap( x11Display(), RootWindow( x11Display(), x11Screen()), visual, AllocNone );
+                attrs.colormap = colormap;
+                flags |= CWColormap;
+                XFree( info );
+                break;
+            }
         }
     }
 #endif
@@ -668,7 +709,7 @@ bool LockProcess::startSaver()
 void LockProcess::stopSaver()
 {
     kdDebug(1204) << "LockProcess: stopping saver" << endl;
-    resume();
+    resume( true );
     stopHack();
     hideSaverWindow();
     mVisibility = false;
@@ -839,14 +880,13 @@ void LockProcess::suspend()
     mSuspended = true;
 }
 
-void LockProcess::resume()
+void LockProcess::resume( bool force )
 {
-    if (!mDialogs.isEmpty())
-        return; // no resuming with dialog visible
-    if(!mVisibility)
-        return; // no need to resume, not visible
+    if( !force && (!mDialogs.isEmpty() || !mVisibility ))
+        return; // no resuming with dialog visible or when not visible
     if(mSuspended)
     {
+        XForceScreenSaver(qt_xdisplay(), ScreenSaverReset );
         bitBlt( this, 0, 0, &mSavedScreen );
         QApplication::syncX();
         mHackProc.kill(SIGCONT);
@@ -861,7 +901,9 @@ void LockProcess::resume()
 //
 bool LockProcess::checkPass()
 {
-    killTimer(mAutoLogoutTimerId);
+    if (mAutoLogout)
+        killTimer(mAutoLogoutTimerId);
+
     PasswordDlg passDlg( this, &greetPlugin);
 
     int ret = execDialog( &passDlg );
@@ -916,7 +958,7 @@ int LockProcess::execDialog( QDialog *dlg )
     if( mDialogs.isEmpty() ) {
         XChangeActivePointerGrab( qt_xdisplay(), GRABEVENTS,
                 blankCursor.handle(), CurrentTime);
-        resume();
+        resume( false );
     } else
         fakeFocusIn( mDialogs.first()->winId());
     return rt;
@@ -973,7 +1015,7 @@ bool LockProcess::x11Event(XEvent *event)
                 else
                 {
                     mSuspendTimer.stop();
-                    resume();
+                    resume( false );
                 }
                 if (event->xvisibility.state != VisibilityUnobscured)
                     stayOnTop();
@@ -1041,9 +1083,13 @@ void LockProcess::checkDPMSActive()
     BOOL on;
     CARD16 state;
     DPMSInfo(qt_xdisplay(), &state, &on);
+    //kdDebug() << "checkDPMSActive " << on << " " << state << endl;
     if (state == DPMSModeStandby || state == DPMSModeSuspend || state == DPMSModeOff)
     {
        suspend();
+    } else if ( mSuspended )
+    {
+        resume( true );
     }
 #endif
 }
