@@ -17,24 +17,45 @@
    Boston, MA 02110-1301, USA.
 */
 
+#include <config.h>
+
 #include "startupid.h"
 #include "klaunchsettings.h"
 
 #include <kiconloader.h>
 #include <qcursor.h>
-#include <qapplication.h>
+#include <kapplication.h>
 #include <qimage.h>
 #include <qbitmap.h>
 #include <kconfig.h>
 #include <X11/Xlib.h>
 
-StartupId::StartupId( QObject* parent, const char* name )
-    :   QObject( parent, name ),
+#define KDE_STARTUP_ICON "kmenu"
+
+#ifdef HAVE_XCURSOR
+#include <X11/Xcursor/Xcursor.h>
+#endif
+
+enum kde_startup_status_enum { StartupPre, StartupIn, StartupDone };
+static kde_startup_status_enum kde_startup_status = StartupPre;
+static Atom kde_splash_progress;
+
+StartupId::StartupId( QWidget* parent, const char* name )
+    :   QWidget( parent, name ),
 	startup_info( KStartupInfo::CleanOnCantDetect ),
 	startup_widget( NULL ),
 	blinking( true ),
 	bouncing( false )
     {
+    hide(); // is QWidget only because of x11Event()
+    if( kde_startup_status == StartupPre )
+        {
+        kde_splash_progress = XInternAtom( qt_xdisplay(), "_KDE_SPLASH_PROGRESS", False );
+        XWindowAttributes attrs;
+        XGetWindowAttributes( qt_xdisplay(), qt_xrootwin(), &attrs);
+        XSelectInput( qt_xdisplay(), qt_xrootwin(), attrs.your_event_mask | SubstructureNotifyMask);
+        kapp->installX11EventFilter( this );
+        }
     connect( &update_timer, SIGNAL( timeout()), SLOT( update_startupid()));
     connect( &startup_info,
         SIGNAL( gotNewStartup( const KStartupInfoId&, const KStartupInfoData& )),
@@ -85,14 +106,45 @@ void StartupId::gotRemoveStartup( const KStartupInfoId& id_P )
     startups.remove( id_P );
     if( startups.count() == 0 )
         {
-        stop_startupid();
         current_startup = KStartupInfoId(); // null
+        if( kde_startup_status == StartupIn )
+            start_startupid( KDE_STARTUP_ICON );
+        else
+            stop_startupid();
         return;
         }
     current_startup = startups.begin().key();
     start_startupid( startups[ current_startup ] );
     }
 
+bool StartupId::x11Event( XEvent* e )
+    {
+    if( e->type == ClientMessage && e->xclient.window == qt_xrootwin()
+        && e->xclient.message_type == kde_splash_progress )
+        {
+        const char* s = e->xclient.data.b;
+        if( strcmp( s, "kicker" ) == 0 && kde_startup_status == StartupPre )
+            {
+            kde_startup_status = StartupIn;
+            if( startups.count() == 0 )
+                start_startupid( KDE_STARTUP_ICON );
+            // 60(?) sec timeout - shouldn't be hopefully needed anyway, ksmserver should have it too
+            QTimer::singleShot( 60000, this, SLOT( finishKDEStartup()));
+            }
+        else if( strcmp( s, "session ready" ) == 0 && kde_startup_status < StartupDone )
+            QTimer::singleShot( 2000, this, SLOT( finishKDEStartup()));
+        }
+    return false;
+    }
+
+void StartupId::finishKDEStartup()
+    {
+    kde_startup_status = StartupDone;
+    kapp->removeX11EventFilter( this );
+    if( startups.count() == 0 )
+        stop_startupid();
+    }
+
 void StartupId::stop_startupid()
     {
     delete startup_widget;
@@ -223,6 +275,20 @@ void StartupId::update_startupid()
         return;
         }
     QPoint c_pos( x, y );
+    int cursor_size = 0;
+#ifdef HAVE_XCURSOR
+    cursor_size = XcursorGetDefaultSize( qt_xdisplay());
+#endif
+    int X_DIFF;
+    if( cursor_size <= 16 )
+        X_DIFF = 8 + 7;
+    else if( cursor_size <= 32 )
+        X_DIFF = 16 + 7;
+    else if( cursor_size <= 48 )
+        X_DIFF = 24 + 7;
+    else
+        X_DIFF = 32 + 7;
+    int Y_DIFF = X_DIFF;
     if( startup_widget->x() != c_pos.x() + X_DIFF
         || startup_widget->y() != c_pos.y() + Y_DIFF + yoffset )
         startup_widget->move( c_pos.x() + X_DIFF, c_pos.y() + Y_DIFF + yoffset );
