@@ -674,6 +674,7 @@ KStdGreeter::KStdGreeter()
 		                 pluginList, KGreeterPlugin::Authenticate,
 		                 KGreeterPlugin::Login );
 	inner_box->addLayout( sverify->getLayout() );
+	QPopupMenu *plugMenu = sverify->getPlugMenu();
 	sverify->selectPlugin( curPlugin );
 	verify = sverify;
 
@@ -690,7 +691,6 @@ KStdGreeter::KStdGreeter()
 		needSep = true;
 	}
 
-	QPopupMenu *plugMenu = verify->getPlugMenu();
 	if (plugMenu) {
 		inserten( i18n("&Authentication Method"), ALT+Key_A, plugMenu );
 		needSep = true;
@@ -758,6 +758,7 @@ KThemedGreeter::KThemedGreeter()
 
 	themer = new KdmThemer( _theme, "console", this );
 	if (!themer->isOK()) {
+		delete themer;
 		themer = 0;
 		return;
 	}
@@ -809,6 +810,7 @@ KThemedGreeter::KThemedGreeter()
 		new KGThemedVerify( this, themer, this, prec, QString::null,
 		                    pluginList, KGreeterPlugin::Authenticate,
 		                    KGreeterPlugin::Login );
+	QPopupMenu *plugMenu = tverify->getPlugMenu();
 	tverify->selectPlugin( curPlugin );
 	verify = tverify;
 
@@ -829,7 +831,6 @@ KThemedGreeter::KThemedGreeter()
 		}
 	}
 
-	QPopupMenu *plugMenu = verify->getPlugMenu();
 	if (plugMenu) {
 		inserten( i18n("&Authentication Method"), ALT+Key_A, plugMenu );
 		needSep = true;
