@@ -175,13 +175,13 @@ static bool xtest()
 #endif
 
 // CHECKME nevola XFlush(), musi se pak volat rucne
-bool Kbd::send_macro_key( unsigned int keycode, Window window_P )
+bool Kbd::send_macro_key( const KKey& key, Window window_P )
     {
-    unsigned int keysym = KKeyNative( KKey( keycode )).sym();
+    unsigned int keysym = KKeyNative( key ).sym();
     KeyCode x_keycode = XKeysymToKeycode( qt_xdisplay(), keysym );
     if( x_keycode == NoSymbol )
 	return false;
-    unsigned int x_mod = KKeyNative( KKey( keycode )).mod();
+    unsigned int x_mod = KKeyNative( key ).mod();
 #ifdef HAVE_XTEST
     if( xtest() && window_P == None )
         {
