@@ -42,7 +42,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 extern "C"
 {
-  KPanelApplet* init(QWidget *parent, const QString& configFile)
+  KDE_EXPORT KPanelApplet* init(QWidget *parent, const QString& configFile)
   {
     KGlobal::locale()->insertCatalogue("krunapplet");
     return new RunApplet(configFile, KPanelApplet::Stretch, 0, parent, "krunapplet");
@@ -75,7 +75,7 @@ RunApplet::RunApplet(const QString& configFile, Type type, int actions,
     _input = new KHistoryCombo(this);
     _input->setFocus();
     _input->clearEdit();
-    _input->lineEdit()->installEventFilter( this );
+    watchForFocus(_input->lineEdit());
     connect(_input, SIGNAL(activated(const QString&)),
 	    SLOT(run_command(const QString&)));
 
@@ -113,15 +113,6 @@ RunApplet::~RunApplet()
     KGlobal::locale()->removeCatalogue("krunapplet");
 }
 
-
-bool RunApplet::eventFilter( QObject *o, QEvent * e)
-{
-    if ( e->type() == QEvent::MouseButtonRelease ) {
-	emit requestFocus();
-    }
-    return KPanelApplet::eventFilter( o, e );
-}
-
 void RunApplet::resizeEvent(QResizeEvent*)
 {
     if(orientation() == Horizontal)
@@ -173,7 +164,7 @@ void RunApplet::setButtonText()
 {
     QString t;
 
-    if (position() == pLeft)
+    if (position() == pRight)
 	{
 	    if (width() >= 42)
 		t = i18n("< Run");
@@ -204,7 +195,7 @@ int RunApplet::heightForWidth(int ) const
 void RunApplet::popup_combo()
 {
     QPoint p;
-    if (position() == pLeft)
+    if (position() == pRight)
 	p = mapToGlobal(QPoint(-_input->width()-1, 0));
     else
 	p = mapToGlobal(QPoint(width()+1, 0));
@@ -216,6 +207,7 @@ void RunApplet::popup_combo()
 void RunApplet::run_command(const QString& command)
 {
     QString exec;
+    bool focusNeeded = false;
 
     kapp->propagateSessionManager();
 
@@ -233,7 +225,7 @@ void RunApplet::run_command(const QString& command)
     if ( cmd.isEmpty() ){
 	KMessageBox::sorry(0L, i18n("You have to enter a command to execute "
 				    "or a URL to be opened first."));
-        emit requestFocus();
+        focusNeeded = true;
 	goto hide;
     }
     else if (cmd == "logout")
@@ -242,11 +234,11 @@ void RunApplet::run_command(const QString& command)
 	    if( !shutdown )
             {
                 // This i18n string is in kdesktop/desktop.cc as well. Maybe we should DCOP to kdesktop instead ?
-		KMessageBox::error( 0, i18n("Unable to logout properly.\nThe session manager cannot "
+		KMessageBox::error( 0, i18n("Unable to log out properly.\nThe session manager cannot "
 					    "be contacted. You can try to force a shutdown by pressing "
 					    "Ctrl+Alt+Backspace. Note, however, that your current "
                                             "session will not be saved with a forced shutdown." ) );
-                emit requestFocus();
+                focusNeeded = true;
             }
 	    goto hide;
 	}
@@ -277,7 +269,7 @@ void RunApplet::run_command(const QString& command)
 						"cannot be found. Please correct the command\n"
 						"or URL and try again</qt>").arg( cmd ) );
 		    _input->removeFromHistory( _input->currentText() );
-                    emit requestFocus();
+                    focusNeeded = true;
 		    goto hide;
 		}
 	}
@@ -288,13 +280,15 @@ void RunApplet::run_command(const QString& command)
 	    KMessageBox::sorry( 0, i18n("<qt>Could not run <b>%1</b>.\nPlease correct"
 					" the command or URL and try again.</qt>").arg( cmd ) );
 	    _input->removeFromHistory( _input->currentText() );
-            emit requestFocus();
+            focusNeeded = true;
 	    goto hide;
 	}
 
+    needsFocus(focusNeeded);
     return;
 
  hide:
     if (orientation() == Vertical)
 	_hbox->hide();
+    needsFocus(focusNeeded);
 }
