@@ -61,8 +61,7 @@ public:
     bool sortByDesktop() const { return m_sortByDesktop; }
     bool showAllWindows() const { return m_showAllWindows; }
 
-    void drawShadowText(QPainter  &p, QRect tr, int tf, const QString & str,
-                        int len = -1, QRect* brect = 0, QTextParag** internal = 0);
+    void drawShadowText(QPainter &p, const QRect &tr, int tf, const QString &str, const QSize &size);
 
     QImage* blendGradient(const QSize& size);
 
@@ -117,6 +116,7 @@ private:
     ArrowType 			arrowType;
     TaskContainer::List		containers;
     TaskContainer::List		m_hiddenContainers;
+    TaskContainer::List		m_deletableContainers;
     PixmapList			frames;
     int                         maximumButtonsWithoutShrinking() const;
     bool                        shouldGroup() const;
