@@ -66,7 +66,8 @@ TaskContainer::TaskContainer(Task::Ptr task, TaskBar* bar,
       taskBar(bar),
       discardNextMouseEvent(false),
       aboutToActivate(false),
-      m_mouseOver(false)
+      m_mouseOver(false),
+      m_paintEventCompression(false)
 {
     init();
     setAcceptDrops(true); // Always enabled to activate task during drag&drop.
@@ -93,7 +94,8 @@ TaskContainer::TaskContainer(Startup::Ptr startup, PixmapList& startupFrames,
       taskBar(bar),
       discardNextMouseEvent(false),
       aboutToActivate(false),
-      m_mouseOver(false)
+      m_mouseOver(false),
+      m_paintEventCompression(false)
 {
     init();
     setEnabled(false);
@@ -116,6 +118,7 @@ void TaskContainer::init()
     connect(&animationTimer, SIGNAL(timeout()), SLOT(animationTimerFired()));
     connect(&dragSwitchTimer, SIGNAL(timeout()), SLOT(showMe()));
     connect(&attentionTimer, SIGNAL(timeout()), SLOT(attentionTimerFired()));
+    connect(&m_paintEventCompressionTimer, SIGNAL(timeout()), SLOT(setPaintEventCompression()));
 }
 
 TaskContainer::~TaskContainer()
@@ -146,8 +149,16 @@ void TaskContainer::stopTimers()
     attentionTimer.stop();
 }
 
-void TaskContainer::taskChanged()
+void TaskContainer::taskChanged(bool geometryOnlyChange)
 {
+    if (geometryOnlyChange)
+    {
+        // we really don't care about those changes, which we may be getting
+        // thanks to the pager, for instance, turning it on in taskmanager.
+        // // let's ignore them so we don't end up with tons of processing going on
+        return;
+    }
+
     const QObject* source = sender();
     Task::Ptr task = 0;
     Task::List::const_iterator itEnd = tasks.constEnd();
@@ -268,10 +279,20 @@ void TaskContainer::checkAttention(const Task::Ptr t)
 void TaskContainer::attentionTimerFired()
 {
     assert( attentionState != -1 );
-    if( attentionState < ATTENTION_BLINK_TIMEOUT )
+    if (attentionState < TaskBarSettings::attentionBlinkIterations()*2)
+    {
         ++attentionState;
-    else
+    }
+    else if (TaskBarSettings::attentionBlinkIterations() < 1000)
+    {
         attentionTimer.stop();
+    }
+    else
+    {
+        // we have a "forever" blinker (attentionBlinkIterations > 999) and have reached
+        // the upper limit. so we need to decrement the attentionState to make it blink
+        --attentionState;
+    }
     update();
 }
 
@@ -307,7 +328,7 @@ void TaskContainer::add(Task::Ptr task)
     KickerTip::Client::updateKickerTip();
     update();
 
-    connect(task, SIGNAL(changed()), SLOT(taskChanged()));
+    connect(task, SIGNAL(changed(bool)), SLOT(taskChanged(bool)));
     connect(task, SIGNAL(iconChanged()), SLOT(iconChanged()));
     connect(task, SIGNAL(activated()), SLOT(setLastActivated()));
 }
@@ -406,8 +427,24 @@ QString TaskContainer::id()
     return sid;
 }
 
+void TaskContainer::setPaintEventCompression()
+{
+    m_paintEventCompression = true;
+    update();
+}
+
 void TaskContainer::paintEvent(QPaintEvent*)
 {
+    if (!m_paintEventCompression)
+    {
+        if (!m_paintEventCompressionTimer.isActive())
+        {
+            m_paintEventCompressionTimer.start(30, true);
+        }
+        return;
+    }
+
+    m_paintEventCompression = false;
     QPixmap* pm = new QPixmap(size());
 
     const QPixmap* background = taskBar->backgroundPixmap();
@@ -499,7 +536,7 @@ void TaskContainer::drawButton(QPainter *p)
 
         if (task->demandsAttention())
         {
-            demandsAttention = attentionState == ATTENTION_BLINK_TIMEOUT ||
+            demandsAttention = attentionState == TaskBarSettings::attentionBlinkIterations() ||
                                attentionState % 2 == 0;
         }
     }
@@ -599,7 +636,7 @@ void TaskContainer::drawButton(QPainter *p)
     // modified overlay
     static QString modStr = "[" + i18n( "modified" ) + "]";
     int modStrPos = text.find( modStr );
-    int textPos = ( taskBar->showIcon() && !pixmap.isNull() ) ? 2 + 16 + 2 : 0;
+    int textPos = ( taskBar->showIcon() && (!pixmap.isNull() || m_startup)) ? 2 + 16 + 2 : 0;
 
     if (modStrPos >= 0)
     {
@@ -670,7 +707,7 @@ void TaskContainer::drawButton(QPainter *p)
 
             if (halo)
             {
-                taskBar->drawShadowText(tp, tr, textFlags, text);
+                taskBar->drawShadowText(tp, tr, textFlags, text, size());
             }
             else
             {
@@ -690,7 +727,7 @@ void TaskContainer::drawButton(QPainter *p)
 
             if (halo)
             {
-                taskBar->drawShadowText(*p, tr, textFlags, text);
+                taskBar->drawShadowText(*p, tr, textFlags, text, size());
             }
             else
             {
@@ -1003,6 +1040,9 @@ void TaskContainer::performAction(int action)
     case TaskBarSettings::Minimize:
         m_filteredTasks.first()->toggleIconified();
     break;
+    case TaskBarSettings::Close:
+        m_filteredTasks.first()->close();
+    break;
     case TaskBarSettings::ToCurrentDesktop:
         m_filteredTasks.first()->toCurrentDesktop();
     break;
@@ -1297,19 +1337,14 @@ void TaskContainer::enterEvent(QEvent* e)
     {
         QString tooltip = "<qt>" + QStyleSheet::escape(name()) + "</qt>";
         QToolTip::add(this, tooltip);
-        QToolButton::enterEvent(e);
         return;
     }
-
-    QToolButton::enterEvent(e);
 }
 
-void TaskContainer::leaveEvent(QEvent* e)
+void TaskContainer::leaveEvent(QEvent*)
 {
     m_mouseOver = false;
     update();
-
-    QToolButton::leaveEvent(e);
 }
 
 void TaskContainer::dragSwitch()
@@ -1481,6 +1516,7 @@ void TaskContainer::updateKickerTip(KickerTip::Data& data)
     if (m_startup)
     {
         data.message = m_startup->text();
+        data.duration = 4000;
         data.subtext = i18n("Loading application ...");
         data.icon = KGlobal::iconLoader()->loadIcon(m_startup->icon(),
                                                     KIcon::Small,
@@ -1579,3 +1615,26 @@ void TaskContainer::updateKickerTip(KickerTip::Data& data)
     data.icon = pixmap;
     data.direction = KickerLib::arrowToDirection(arrowType);
 }
+
+void TaskContainer::finish()
+{
+    // Disconnect all signal/slot connections to avoid triggering a popupMenu() call,
+    // whose event loop is the root of all (or at least much) evil.
+    // Unfortunately, we can't just do "disconnect()", because that gets us a bunch
+    // of dangling QGuardedPtr objects (most notably in QTipManager.) (kling)
+
+    animationTimer.disconnect();
+    dragSwitchTimer.disconnect();
+    attentionTimer.disconnect();
+
+    if (m_startup)
+        m_startup->disconnect(this);
+
+    for (Task::List::Iterator it = tasks.begin(); it != tasks.end(); ++it)
+    {
+        (*it)->disconnect(this);
+    }
+
+    if (m_menu)
+        m_menu->close();
+}
