@@ -13,8 +13,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #ifndef KIO_FLOPPY_H
@@ -65,6 +65,7 @@ class FloppyProtocol : public KIO::SlaveBase
       int freeSpace(const KURL& url);
 
       bool stopAfterError(const KURL& url, const QString& drive);
+      void errorMissingMToolsProgram(const QString& name);
 
       void clearBuffers();
       void terminateBuffers();
