@@ -21,6 +21,8 @@
 
 #include <stdlib.h>
 
+#include <kapplication.h>
+#include <qeventloop.h>
 #include <qfile.h>
 #include <klocale.h>
 #include <kurl.h>
@@ -43,8 +45,11 @@ QString libhal_device_get_property_QString(LibHalContext *ctx, const char* udi,
 {
     char*   _ppt_string;
     QString _ppt_QString;
-    _ppt_string = libhal_device_get_property_string(ctx, udi, key, NULL);
-    _ppt_QString = QString(_ppt_string ? _ppt_string : "");
+    DBusError error;
+    dbus_error_init(&error);
+    _ppt_string = libhal_device_get_property_string(ctx, udi, key, &error);
+    if ( _ppt_string )
+        _ppt_QString = _ppt_string;
     libhal_free_string(_ppt_string);
     return _ppt_QString;
 }
@@ -111,7 +116,7 @@ bool HALBackend::InitHal()
     kdDebug(1219) << "Main loop integration" << endl;
     DBusError error;
     dbus_error_init(&error);
-    dbus_connection = dbus_connection_open_private(DBUS_SYSTEM_BUS, &error);
+    dbus_connection = dbus_bus_get_private(DBUS_BUS_SYSTEM, &error);
 
     if (!dbus_connection || dbus_error_is_set(&error)) {
         dbus_error_free(&error);
@@ -120,14 +125,8 @@ bool HALBackend::InitHal()
         return false;
     }
 
-    dbus_bus_register(dbus_connection, &error);
+    dbus_connection_set_exit_on_disconnect (dbus_connection, FALSE);
 
-    if (dbus_error_is_set(&error)) {
-        dbus_error_free(&error);
-        libhal_ctx_free(m_halContext);
-        m_halContext = NULL;
-        return false;
-    }
     MainLoopIntegration(dbus_connection);
     libhal_ctx_set_dbus_connection(m_halContext, dbus_connection);
 
@@ -215,13 +214,11 @@ void HALBackend::AddDevice(const char *udi, bool allowNotification)
              !libhal_device_get_property_bool(m_halContext, udi, "volume.disc.is_blank", NULL) )
             return;
 
-        if ( ( libhal_device_get_property_QString(m_halContext, udi, "volume.fsusage") == "filesystem" &&
-               !libhal_device_get_property_bool(m_halContext, udi, "volume.is_mounted", NULL ) ) &&
-             ( libhal_device_get_property_bool(m_halContext, udi, "volume.ignore", NULL ) ) )
-            return;
-
         /* Query drive udi */
         QString driveUdi = libhal_device_get_property_QString(m_halContext, udi, "block.storage_device");
+        if ( driveUdi.isNull() ) // no storage - no fun
+            return;
+
         /* We don't list floppy volumes because we list floppy drives */
         if ((libhal_device_get_property_QString(m_halContext, driveUdi.ascii(), "storage.drive_type") == "floppy") ||
             (libhal_device_get_property_QString(m_halContext, driveUdi.ascii(), "storage.drive_type") == "zip") ||
@@ -240,7 +237,19 @@ void HALBackend::AddDevice(const char *udi, bool allowNotification)
 
         /* Create medium */
         Medium* medium = new Medium(udi, "");
+        setVolumeProperties(medium);
 
+        if ( isInFstab( medium ).isNull() )
+        {
+            // if it's not mountable by user and not by HAL, don't show it at all
+            if ( ( libhal_device_get_property_QString(m_halContext, udi, "volume.fsusage") == "filesystem" &&
+                   !libhal_device_get_property_bool(m_halContext, udi, "volume.is_mounted", NULL ) ) &&
+                 ( libhal_device_get_property_bool(m_halContext, udi, "volume.ignore", NULL ) ) )
+            {
+                delete medium;
+                return;
+            }
+        }
         QMap<QString,QString> options = MediaManagerUtils::splitOptions(mountoptions(udi));
         kdDebug() << "automount " << options["automount"] << endl;
         if (options["automount"] == "true" && allowNotification ) {
@@ -248,7 +257,6 @@ void HALBackend::AddDevice(const char *udi, bool allowNotification)
             if (!error.isEmpty())
                 kdDebug() << "error " << error << endl;
         }
-        setVolumeProperties(medium);
         m_mediaList.addMedium(medium, allowNotification);
 
         return;
@@ -388,7 +396,7 @@ void HALBackend::ResetProperties(const char* mediumUdi)
                 m_mediaList.changeMediumState(m, false);
             }
             return;
-        }
+       }
     }
 
     Medium* m = new Medium(mediumUdi, "");
@@ -419,7 +427,9 @@ void HALBackend::setVolumeProperties(Medium* medium)
     if (!halVolume)
         return;
     QString driveUdi = libhal_volume_get_storage_device_udi(halVolume);
-    LibHalDrive*  halDrive  = libhal_drive_from_udi(m_halContext, driveUdi.ascii());
+    LibHalDrive*  halDrive  = 0;
+    if ( !driveUdi.isNull() )
+        halDrive = libhal_drive_from_udi(m_halContext, driveUdi.ascii());
     if (!halDrive) {
         // at times HAL sends an UnmountForced event before the device is removed
         libhal_volume_free(halVolume);
@@ -495,6 +505,7 @@ void HALBackend::setVolumeProperties(Medium* medium)
     else
     {
         mimeType = "media/hdd" + MOUNT_SUFFIX;
+        medium->setIconName(QString::null); // reset icon
         if (libhal_drive_is_hotpluggable(halDrive))
         {
             mimeType = "media/removable" + MOUNT_SUFFIX;
@@ -520,7 +531,6 @@ void HALBackend::setVolumeProperties(Medium* medium)
             case LIBHAL_DRIVE_TYPE_CAMERA:
             {
                 mimeType = "media/camera" + MOUNT_SUFFIX;
-                medium->setIconName("camera" + MOUNT_ICON_SUFFIX);
                 const char *physdev = libhal_drive_get_physical_device_udi(halDrive);
                 // get model from camera
                 if (physdev && libhal_device_query_capability(m_halContext, physdev, "camera", NULL))
@@ -542,7 +552,6 @@ void HALBackend::setVolumeProperties(Medium* medium)
             if (medium->isMounted() && QFile::exists(medium->mountPoint() + "/dcim"))
             {
                 mimeType = "media/camera" + MOUNT_SUFFIX;
-                medium->setIconName("camera" + MOUNT_ICON_SUFFIX);
             }
         }
     }
@@ -628,7 +637,7 @@ void HALBackend::setFloppyProperties(Medium* medium)
     {
         // HAL hates floppies - so we have to do it twice ;(
         medium->mountableState(libhal_drive_get_device_file(halDrive), QString::null, QString::null, false);
-        setFstabProperties( medium );
+        setFloppyMountState(medium);
     }
 
     if (libhal_device_get_property_QString(m_halContext, udi, "storage.drive_type") == "floppy")
@@ -663,6 +672,29 @@ void HALBackend::setFloppyProperties(Medium* medium)
     libhal_volume_free(halVolume);
 }
 
+void HALBackend::setFloppyMountState( Medium *medium )
+{
+    if ( !medium->id().startsWith( "/org/kde" ) )
+    {
+        KMountPoint::List mtab = KMountPoint::currentMountPoints();
+        KMountPoint::List::iterator it = mtab.begin();
+        KMountPoint::List::iterator end = mtab.end();
+
+        QString fstype;
+        QString mountpoint;
+        for (; it!=end; ++it)
+        {
+            if ((*it)->mountedFrom() == medium->deviceNode() )
+            {
+                fstype = (*it)->mountType().isNull() ? (*it)->mountType() : "auto";
+                mountpoint = (*it)->mountPoint();
+                medium->mountableState( medium->deviceNode(), mountpoint, fstype, true );
+                return;
+            }
+        }
+    }
+}
+
 void HALBackend::setCameraProperties(Medium* medium)
 {
     kdDebug(1219) << "HALBackend::setCameraProperties for " << medium->id() << endl;
@@ -772,10 +804,11 @@ QStringList HALBackend::mountoptions(const QString &name)
     if (fstype.isNull())
         fstype = libhal_device_get_property_QString(m_halContext, name.latin1(), "volume.policy.mount_filesystem");
 
-
     QString drive_udi = libhal_device_get_property_QString(m_halContext, name.latin1(), "block.storage_device");
 
-    bool removable = libhal_device_get_property_bool(m_halContext, drive_udi.latin1(), "storage.removable", NULL)
+    bool removable = false;
+    if ( !drive_udi.isNull() )
+        removable = libhal_device_get_property_bool(m_halContext, drive_udi.latin1(), "storage.removable", NULL)
                      || libhal_device_get_property_bool(m_halContext, drive_udi.latin1(), "storage.hotpluggable", NULL);
 
     config.setGroup(drive_udi);
@@ -972,15 +1005,79 @@ static QString mount_priv(const char *udi, const char *mount_point, const char *
 
 }
 
+QString HALBackend::listUsingProcesses(const Medium* medium)
+{
+    QString proclist, fullmsg;
+    QString cmdline = QString("/usr/bin/env fuser -vm %1 2>&1").arg(KProcess::quote(medium->mountPoint()));
+    FILE *fuser = popen(cmdline.latin1(), "r");
+
+    uint counter = 0;
+    if (fuser) {
+        proclist += "<pre>";
+        QTextIStream is(fuser);
+        QString tmp;
+        while (!is.atEnd()) {
+            tmp = is.readLine();
+            tmp = QStyleSheet::escape(tmp) + "\n";
+
+            proclist += tmp;
+            if (counter++ > 10)
+            {
+                proclist += "...";
+                break;
+            }
+        }
+        proclist += "</pre>";
+        (void)pclose( fuser );
+    }
+    if (counter) {
+        fullmsg = i18n("Moreover, programs still using the device "
+            "have been detected. They are listed below. You have to "
+            "close them or change their working directory before "
+            "attempting to unmount the device again.");
+        fullmsg += "<br>" + proclist;
+        return fullmsg;
+    } else {
+        return QString::null;
+    }
+}
+
 void HALBackend::slotResult(KIO::Job *job)
 {
     kdDebug() << "slotResult " << mount_jobs[job] << endl;
-    if (job->error())
-    {
-        KMessageBox::error(0, job->errorText());
+
+    struct mount_job_data *data = mount_jobs[job];
+    QString& qerror = data->errorMessage;
+    const Medium* medium = data->medium;
+
+    if (job->error() == KIO::ERR_COULD_NOT_UNMOUNT) {
+        QString proclist(listUsingProcesses(medium));
+
+        qerror = "<qt>";
+        qerror += i18n("Unfortunately, the device <b>%1</b> (%2) named <b>'%3'</b> and "
+                       "currently mounted at <b>%4</b> could not be unmounted. ").arg(
+                          "system:/media/" + medium->name(),
+                          medium->deviceNode(),
+                          medium->prettyLabel(),
+                          medium->prettyBaseURL().pathOrURL());
+        qerror += i18n("The following error was returned by umount command:");
+        qerror += "<br><pre>" + job->errorText() + "</pre>";
+
+        if (!proclist.isEmpty()) {
+            qerror += proclist;
+        }
+        qerror += "</qt>";
+    } else if (job->error()) {
+        qerror = job->errorText();
     }
-    ResetProperties( mount_jobs[job].latin1() );
+
+    ResetProperties( medium->id().latin1() );
     mount_jobs.remove(job);
+
+    /* Job completed. Notify the caller */
+    data->error = job->error();
+    data->completed = true;
+    kapp->eventLoop()->exitLoop();
 }
 
 QString HALBackend::isInFstab(const Medium *medium)
@@ -992,8 +1089,11 @@ QString HALBackend::isInFstab(const Medium *medium)
 
     for (; it!=end; ++it)
     {
-        kdDebug() << "isInFstab -" << medium->deviceNode() << "- -" << (*it)->realDeviceName() << "- -" << (*it)->mountedFrom() << endl;
-        if ((*it)->mountedFrom() == medium->deviceNode() || ( !medium->deviceNode().isEmpty() && (*it)->realDeviceName() == medium->deviceNode() ) )
+        QString reald = (*it)->realDeviceName();
+        if ( reald.endsWith( "/" ) )
+            reald = reald.left( reald.length() - 1 );
+        kdDebug() << "isInFstab -" << medium->deviceNode() << "- -" << reald << "- -" << (*it)->mountedFrom() << "-" << endl;
+        if ((*it)->mountedFrom() == medium->deviceNode() || ( !medium->deviceNode().isEmpty() && reald == medium->deviceNode() ) )
 	{
             QStringList opts = (*it)->mountOptions();
             if (opts.contains("user") || opts.contains("users"))
@@ -1012,14 +1112,25 @@ QString HALBackend::mount(const Medium *medium)
     QString mountPoint = isInFstab(medium);
     if (!mountPoint.isNull())
     {
+        struct mount_job_data data;
+        data.completed = false;
+        data.medium = medium;
+
         kdDebug() << "triggering user mount " << medium->deviceNode() << " " << mountPoint << " " << medium->id() << endl;
         KIO::Job *job = KIO::mount( false, 0, medium->deviceNode(), mountPoint );
         connect(job, SIGNAL( result (KIO::Job *)),
                 SLOT( slotResult( KIO::Job *)));
-        mount_jobs[job] = medium->id();
-        return QString(); // we won't report an error here
+        mount_jobs[job] = &data;
+        // The caller expects the device to be mounted when the function
+        // completes. Thus block until the job completes.
+        while (!data.completed) {
+            kapp->eventLoop()->enterLoop();
+        }
+        // Return the error message (if any) to the caller
+        return (data.error) ? data.errorMessage : QString::null;
+
     } else if (medium->id().startsWith("/org/kde/") )
-	return i18n("Permissions denied");
+	    return i18n("Permissions denied");
 
     QStringList soptions;
 
@@ -1121,12 +1232,22 @@ QString HALBackend::unmount(const QString &_udi)
     QString mountPoint = isInFstab(medium);
     if (!mountPoint.isNull())
     {
+        struct mount_job_data data;
+        data.completed = false;
+        data.medium = medium;
+
         kdDebug() << "triggering user unmount " << medium->deviceNode() << " " << mountPoint << endl;
         KIO::Job *job = KIO::unmount( medium->mountPoint(), false );
         connect(job, SIGNAL( result (KIO::Job *)),
                 SLOT( slotResult( KIO::Job *)));
-        mount_jobs[job] = medium->id();
-        return QString(); // we won't report an error here
+        mount_jobs[job] = &data;
+        // The caller expects the device to be unmounted when the function
+        // completes. Thus block until the job completes.
+        while (!data.completed) {
+            kapp->eventLoop()->enterLoop();
+        }
+        // Return the error message (if any) to the caller
+        return (data.error) ? data.errorMessage : QString::null;
     }
 
     DBusMessage *dmesg, *reply;
@@ -1165,35 +1286,31 @@ QString HALBackend::unmount(const QString &_udi)
     dbus_error_init (&error);
     if (!(reply = dbus_connection_send_with_reply_and_block (dbus_connection, dmesg, -1, &error)))
     {
-        kdDebug() << "unmount failed for " << udi << ": " << error.name << " " << error.message << endl;
-        QString qerror = error.message;
-        if (!strcmp(error.name, "org.freedesktop.Hal.Device.Volume.Busy")) {
-            qerror = QString("<qt>") + i18n("Device is Busy:");
-            QString cmdline = QString("/usr/bin/env fuser -vm %1 2>&1").arg(KProcess::quote(medium->mountPoint()));
-            FILE *fuser = popen(cmdline.latin1(), "r");
-            uint counter = 0;
-            if (fuser) {
-                qerror += "<pre>";
-                QTextIStream is(fuser);
-                QString tmp;
-                while (!is.atEnd()) {
-                    tmp = is.readLine();
-                    tmp = QStyleSheet::escape(tmp) + "\n";
+        QString qerror, reason;
 
-                    qerror += tmp;
-                    if (counter++ > 20)
-                    {
-                        qerror += "...";
-                        break;
-                    }
-                }
-                qerror += "</pre>";
-                (void)pclose( fuser );
-            }
-            qerror += "</qt>";
+        kdDebug() << "unmount failed for " << udi << ": " << error.name << " " << error.message << endl;
+        qerror = "<qt>";
+        qerror += i18n("Unfortunately, the device <b>%1</b> (%2) named <b>'%3'</b> and "
+                       "currently mounted at <b>%4</b> could not be unmounted. ").arg(
+                          "system:/media/" + medium->name(),
+                          medium->deviceNode(),
+                          medium->prettyLabel(),
+                          medium->prettyBaseURL().pathOrURL());
+        qerror += i18n("Unmounting failed due to the following error:");
+        if (!strcmp(error.name, "org.freedesktop.Hal.Device.Volume.Busy")) {
+            reason = i18n("Device is Busy:");
         } else if (!strcmp(error.name, "org.freedesktop.Hal.Device.Volume.NotMounted")) {
             // this is faking. The error is that the device wasn't mounted by hal (but by the system)
-            qerror = i18n("Permissions denied");
+            reason = i18n("Permissions denied");
+        } else {
+            reason = error.message;
+        }
+        qerror += "<p><b>" + reason + "</b></p>";
+
+        // Include list of processes (if any) using the device in the error message
+        reason = listUsingProcesses(medium);
+        if (!reason.isEmpty()) {
+            qerror += reason;
         }
 
         dbus_message_unref (dmesg);
