@@ -147,10 +147,12 @@ private:
 
 	void setVolumeProperties(Medium* medium);
 	void setFloppyProperties(Medium* medium);
+    void setFloppyMountState( Medium* medium );
 	bool setFstabProperties(Medium* medium);
 	void setCameraProperties(Medium* medium);
 	QString generateName(const QString &devNode);
 	static QString isInFstab(const Medium *medium);
+	static QString listUsingProcesses(const Medium *medium);
 
 private slots:
 	void slotResult(KIO::Job *job);
@@ -216,7 +218,21 @@ private:
 
 	DBusConnection *dbus_connection;
 
-	QMap<KIO::Job *, QString> mount_jobs;
+	/**
+	* Data structure for fstab mount/unmount jobs
+	*/
+	struct mount_job_data {
+		// [in] Medium, which is being mounted/unmounted by the job
+		const Medium* medium;
+		// [in,out] Should be set to true when the job completes
+		bool completed;
+		// [out] KIO::Error if an error occured during operation. Otherwise, 0
+		int error;
+		// [out] Error message to be displayed to the user
+		QString errorMessage;
+	};
+
+	QMap<KIO::Job *, struct mount_job_data*> mount_jobs;
 };
 
 #endif /* _HALBACKEND_H_ */
