@@ -47,7 +47,7 @@ int SMBSlave::cache_stat(const SMBUrl &url, struct stat* st )
 {
     int result = smbc_stat( url.toSmbcUrl(), st);
     kdDebug(KIO_SMB) << "smbc_stat " << url << " " << errno << " " << result << endl;
-    kdDebug(KIO_SMB) << "size " << (long)st->st_size << endl;
+    kdDebug(KIO_SMB) << "size " << (KIO::filesize_t)st->st_size << endl;
     return result;
 }
 
@@ -239,7 +239,7 @@ void SMBSlave::reportError(const SMBUrl &url)
     {
     case ENOENT:
         if (url.getType() == SMBURLTYPE_ENTIRE_NETWORK)
-            error( ERR_SLAVE_DEFINED, i18n("Unable to find any workgroups in your local network."));
+            error( ERR_SLAVE_DEFINED, i18n("Unable to find any workgroups in your local network. This might be caused by an enabled firewall."));
         else
             error( ERR_DOES_NOT_EXIST, url.prettyURL());
         break;
@@ -257,7 +257,10 @@ void SMBSlave::reportError(const SMBUrl &url)
                 i18n( "Could not connect to host for %1" ).arg( url.prettyURL() ) );
         break;
     case ENOTDIR:
+        error( ERR_CANNOT_ENTER_DIRECTORY, url.prettyURL());
+        break;
     case EFAULT:
+    case EINVAL:
         error( ERR_DOES_NOT_EXIST, url.prettyURL());
         break;
     case EPERM:
@@ -280,6 +283,9 @@ void SMBSlave::reportError(const SMBUrl &url)
     case EBADF:
         error( ERR_INTERNAL, i18n("BAD File descriptor"));
         break;
+    case ETIMEDOUT:
+        error( ERR_SERVER_TIMEOUT, url.host() );
+        break;
 #ifdef ENOTUNIQ
     case ENOTUNIQ:
         error( ERR_SLAVE_DEFINED, i18n( "The given name could not be resolved to a unique server. "
@@ -338,7 +344,9 @@ void SMBSlave::listDir( const KURL& kurl )
            // Set name
            atom.m_uds = KIO::UDS_NAME;
            QString dirpName = QString::fromUtf8( dirp->name );
-           QString comment = QString::fromUtf8( dirp->comment, dirp->commentlen - 1);
+           // We cannot trust dirp->commentlen has it might be with or without the NUL character
+           // See KDE bug #111430 and Samba bug #3030
+           QString comment = QString::fromUtf8( dirp->comment );
            if ( dirp->smbc_type == SMBC_SERVER || dirp->smbc_type == SMBC_WORKGROUP ) {
                atom.m_str = dirpName.lower();
                atom.m_str.at( 0 ) = dirpName.at( 0 ).upper();
