@@ -30,6 +30,8 @@
 #include <kstandarddirs.h>
 #include <kglobalsettings.h>
 #include <kmountpoint.h>
+#include <kfileitem.h>
+#include <kio/chmodjob.h>
 
 #include <dcopref.h>
 
@@ -181,7 +183,7 @@ void TrashImpl::migrateOldTrash()
     if ( allOK ) {
         // We need to remove the old one, otherwise the desktop will have two trashcans...
         kdDebug() << "Trash migration: all OK, removing old trash directory" << endl;
-        synchronousDel( oldTrashDir, false );
+        synchronousDel( oldTrashDir, false, true );
     }
 }
 
@@ -321,7 +323,7 @@ bool TrashImpl::moveToTrash( const QString& origPath, int trashId, const QString
         if ( QFileInfo( dest ).isFile() )
             QFile::remove( dest );
         else
-            synchronousDel( dest, false );
+            synchronousDel( dest, false, true );
         return false;
     }
     fileAdded();
@@ -475,20 +477,35 @@ bool TrashImpl::del( int trashId, const QString& fileId )
         return false;
     }
 
-    QFile::remove( info );
-
-    if ( !synchronousDel( file, true ) )
+    if ( !synchronousDel( file, true, QFileInfo(file).isDir() ) )
         return false;
+
+    QFile::remove( info );
     fileRemoved();
     return true;
 }
 
-bool TrashImpl::synchronousDel( const QString& path, bool setLastErrorCode )
+bool TrashImpl::synchronousDel( const QString& path, bool setLastErrorCode, bool isDir )
 {
     const int oldErrorCode = m_lastErrorCode;
     const QString oldErrorMsg = m_lastErrorMessage;
     KURL url;
     url.setPath( path );
+
+    // First ensure that all dirs have u+w permissions,
+    // otherwise we won't be able to delete files in them (#130780).
+    if ( isDir ) {
+        kdDebug() << k_funcinfo << "chmod'ing " << url << endl;
+        KFileItem fileItem( url, "inode/directory", KFileItem::Unknown );
+        KFileItemList fileItemList;
+        fileItemList.append( &fileItem );
+        KIO::ChmodJob* chmodJob = KIO::chmod( fileItemList, 0200, 0200, QString::null, QString::null, true /*recursive*/, false /*showProgressInfo*/ );
+        connect( chmodJob, SIGNAL( result(KIO::Job *) ),
+                 this, SLOT( jobFinished(KIO::Job *) ) );
+        qApp->eventLoop()->enterLoop();
+    }
+
+    kdDebug() << k_funcinfo << "deleting " << url << endl;
     KIO::DeleteJob *job = KIO::del( url, false, false );
     connect( job, SIGNAL( result(KIO::Job *) ),
              this, SLOT( jobFinished(KIO::Job *) ) );
@@ -501,24 +518,28 @@ bool TrashImpl::synchronousDel( const QString& path, bool setLastErrorCode )
     return ok;
 }
 
-void TrashImpl::emptyTrash()
+bool TrashImpl::emptyTrash()
 {
     kdDebug() << k_funcinfo << endl;
-    if ( !m_trashDirectoriesScanned )
-        scanTrashDirectories();
-    // For each known trash directory...
-    TrashDirMap::const_iterator it = m_trashDirectories.begin();
-    for ( ; it != m_trashDirectories.end() ; ++it ) {
-        QDir dir;
-        const QString infoPath = it.data() + "/info";
-        // TODO show errors (with warning()), e.g. when permission denied?
-        synchronousDel( infoPath, false );
-        dir.mkdir( infoPath );
-        const QString filesPath = it.data() + "/files";
-        synchronousDel( filesPath, false );
-        dir.mkdir( filesPath );
+    // The naive implementation "delete info and files in every trash directory"
+    // breaks when deleted directories contain files owned by other users.
+    // We need to ensure that the .trashinfo file is only removed when the
+    // corresponding files could indeed be removed.
+
+    const TrashedFileInfoList fileInfoList = list();
+
+    TrashedFileInfoList::const_iterator it = fileInfoList.begin();
+    const TrashedFileInfoList::const_iterator end = fileInfoList.end();
+    for ( ; it != end ; ++it ) {
+        const TrashedFileInfo& info = *it;
+        const QString filesPath = info.physicalPath;
+        if ( synchronousDel( filesPath, true, true ) ) {
+            QFile::remove( infoPath( info.trashId, info.fileId ) );
+        } // else error code is set
     }
     fileRemoved();
+
+    return m_lastErrorCode == 0;
 }
 
 TrashImpl::TrashedFileInfoList TrashImpl::list()
@@ -763,7 +784,7 @@ QString TrashImpl::trashForMountPoint( const QString& topdir, bool createIfNeede
     uid_t uid = getuid();
     KDE_struct_stat buff;
     // Minimum permissions required: write+execute for 'others', and sticky bit
-    int requiredBits = S_IWOTH | S_IXOTH | S_ISVTX;
+    uint requiredBits = S_IWOTH | S_IXOTH | S_ISVTX;
     if ( KDE_lstat( QFile::encodeName( rootTrashDir ), &buff ) == 0 ) {
         if ( (buff.st_uid == 0) // must be owned by root
              && (S_ISDIR(buff.st_mode)) // must be a dir
