@@ -1,4 +1,4 @@
-// -*- mode:cperl; cperl-indent-level:4; cperl-continued-statement-offset:4; indent-tabs-mode:nil -*-
+// -*- indent-tabs-mode:nil -*-
 // vim: set ts=4 sts=4 sw=4 et:
 /* This file is part of the KDE project
    Copyright (C) 2000 David Faure <faure@kde.org>
@@ -15,8 +15,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include "importers.h"
@@ -108,10 +108,6 @@ void ImportCommand::execute() {
         KMacroCommand *mcmd = (KMacroCommand*) m_cleanUpCmd;
         mcmd->addCommand(new DeleteCommand(bkGroup.address(),
                     true /* contentOnly */));
-
-        // unselect current item, it doesn't exist anymore
-        if (ListView::self())
-            ListView::self()->clearSelection();
         m_cleanUpCmd->execute();
 
         // import at the root
@@ -132,9 +128,6 @@ void ImportCommand::unexecute() {
         KBookmarkGroup root = CurrentMgr::self()->mgr()->root();
         KCommand *cmd = DeleteCommand::deleteAll(root);
 
-        // unselect current item, it doesn't exist anymore
-        if (ListView::self())
-            ListView::self()->clearSelection();
         cmd->execute();
         delete cmd;
 
@@ -143,6 +136,15 @@ void ImportCommand::unexecute() {
     }
 }
 
+QString ImportCommand::affectedBookmarks() const
+{
+    QString rootAdr = CurrentMgr::self()->mgr()->root().address();
+    if(m_group == rootAdr)
+        return m_group;
+    else
+        return KBookmark::parentAddress(m_group);
+}
+
 /* -------------------------------------- */
 
 QString MozImportCommand::requestFilename() const {
