@@ -62,7 +62,7 @@ public:
   QPtrList<KonqFrameBase>* childFrameList() { return m_pChildFrameList; }
 
   virtual void setTitle( const QString &title, QWidget* sender );
-  virtual void setTabIcon( const QString &url, QWidget* sender );
+  virtual void setTabIcon( const KURL &url, QWidget* sender );
 
   virtual QWidget* widget() { return this; }
   virtual QCString frameType() { return QCString("Tabs"); }
