@@ -1,14 +1,22 @@
-/* ------------------------------------------------------------------------ */
-/*                                                                          */
-/*                       Terminal Emulation Widget                          */
-/*                                                                          */
-/* ------------------------------------------------------------------------ */
-/*                                                                          */
-/* Copyright (c) 1997,1998 by Lars Doelle <lars.doelle@on-line.de>          */
-/*                                                                          */
-/* This file is part of Konsole - an X terminal for KDE                     */
-/*                                                                          */
-/* ------------------------------------------------------------------------ */
+/*
+    This file is part of Konsole, an X terminal.
+    Copyright (C) 1997,1998 by Lars Doelle <lars.doelle@on-line.de>
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA.
+*/
 
 /*! \class TEWidget
 
@@ -267,7 +275,7 @@ void TEWidget::fontChange(const QFont &)
       break;
   }
   }
-  
+
   if (font_w>200) // don't trust unrealistic value, fallback to QFontMetrics::maxWidth()
     font_w=fm.maxWidth();
   if (font_w<1)
@@ -298,10 +306,17 @@ void TEWidget::fontChange(const QFont &)
 void TEWidget::setVTFont(const QFont& f)
 {
   QFont font = f;
-  if (!s_antialias)
-    font.setStyleStrategy( QFont::NoAntialias );
-  QFrame::setFont(font);
-  fontChange(font);
+
+  // the font must be small enough to allow at least one line and one character of text to fit 
+  // on screen
+  QFontMetrics metrics(f);
+  if ( metrics.height() < height() && metrics.maxWidth() < width() )
+  {
+    if (!s_antialias)
+        font.setStyleStrategy( QFont::NoAntialias );
+    QFrame::setFont(font);
+    fontChange(font);
+  }
 }
 
 void TEWidget::setFont(const QFont &)
@@ -400,8 +415,8 @@ TEWidget::TEWidget(QWidget *parent, const char *name)
   setFocusPolicy( WheelFocus );
 
   // im
-  setInputMethodEnabled(true);  
-  
+  setInputMethodEnabled(true);
+
   if (!argb_visual)
   {
     // Looks better at startup with KRootPixmap based pseudo-transparancy
@@ -428,7 +443,7 @@ TEWidget::~TEWidget()
  It represents the 250x - 257x glyphs. If it's zero, we can't use it.
  if it's not, it's encoded as follows: imagine a 5x5 grid where the points are numbered
  0 to 24 left to top, top to bottom. Each point is represented by the corresponding bit.
-  
+
  Then, the pixels basically have the following interpretation:
  _|||_
  -...-
@@ -440,32 +455,32 @@ where _ = none
       | = vertical line.
       - = horizontal line.
  */
- 
+
 
 enum LineEncode
 {
     TopL  = (1<<1),
     TopC  = (1<<2),
     TopR  = (1<<3),
-    
+
     LeftT = (1<<5),
     Int11 = (1<<6),
     Int12 = (1<<7),
     Int13 = (1<<8),
     RightT = (1<<9),
-    
+
     LeftC = (1<<10),
     Int21 = (1<<11),
     Int22 = (1<<12),
     Int23 = (1<<13),
     RightC = (1<<14),
-    
+
     LeftB = (1<<15),
     Int31 = (1<<16),
     Int32 = (1<<17),
     Int33 = (1<<18),
     RightB = (1<<19),
-    
+
     BotL  = (1<<21),
     BotC  = (1<<22),
     BotR  = (1<<23)
@@ -482,7 +497,7 @@ static void drawLineChar(QPainter& paint, int x, int y, int w, int h, uchar code
     int ey = y + h - 1;
 
     Q_UINT32 toDraw = LineChars[code];
-    
+
     //Top lines:
     if (toDraw & TopL)
         paint.drawLine(cx-1, y, cx-1, cy-2);
@@ -514,7 +529,7 @@ static void drawLineChar(QPainter& paint, int x, int y, int w, int h, uchar code
         paint.drawLine(cx+2, cy, ex, cy);
     if (toDraw & RightB)
         paint.drawLine(cx+2, cy+1, ex, cy+1);
-    
+
     //Intersection points.
     if (toDraw & Int11)
         paint.drawPoint(cx-1, cy-1);
@@ -522,7 +537,7 @@ static void drawLineChar(QPainter& paint, int x, int y, int w, int h, uchar code
         paint.drawPoint(cx, cy-1);
     if (toDraw & Int13)
         paint.drawPoint(cx+1, cy-1);
-    
+
     if (toDraw & Int21)
         paint.drawPoint(cx-1, cy);
     if (toDraw & Int22)
@@ -559,7 +574,7 @@ void TEWidget::drawTextFixed(QPainter &paint, int x, int y,
       w = font_w*2;
       nc+=2;
     }
-    
+
     //Check for line-drawing char
     if (isLineChar(drawstr[0].unicode()))
     {
@@ -571,7 +586,7 @@ void TEWidget::drawTextFixed(QPainter &paint, int x, int y,
             continue;
         }
     }
-    
+
     paint.drawText(x,y, w, font_h, Qt::AlignHCenter | Qt::DontClip, drawstr, -1);
     x += w;
   }
@@ -616,7 +631,7 @@ void TEWidget::drawAttrStr(QPainter &paint, QRect rect,
           Q_UINT8 dalpha = 255 - salpha;
 
           int a, r, g, b;
-          a = QMIN( (qAlpha (col) * salpha) / 255 + (qAlpha (blend_color) * dalpha) / 255, 255 ); 
+          a = QMIN( (qAlpha (col) * salpha) / 255 + (qAlpha (blend_color) * dalpha) / 255, 255 );
           r = QMIN( (qRed   (col) * salpha) / 255 + (qRed   (blend_color) * dalpha) / 255, 255 );
           g = QMIN( (qGreen (col) * salpha) / 255 + (qGreen (blend_color) * dalpha) / 255, 255 );
           b = QMIN( (qBlue  (col) * salpha) / 255 + (qBlue  (blend_color) * dalpha) / 255, 255 );
@@ -646,7 +661,7 @@ void TEWidget::drawAttrStr(QPainter &paint, QRect rect,
       int w = font_w * (m_imSelEnd - m_imSelStart);
       int h = font_h;
 
-      QRect tmpRect = QRect( x, y, w, h ); 
+      QRect tmpRect = QRect( x, y, w, h );
       if ( str != m_imPreeditText ) {  // ugly hack
         tmpRect.setLeft( tmpRect.left() + font_w );
         tmpRect.setWidth( tmpRect.width() + font_w );
@@ -676,7 +691,6 @@ void TEWidget::drawAttrStr(QPainter &paint, QRect rect,
     }
   }
 
-  // Paint text
   if (!(blinking && (attr->r & RE_BLINK)))
   {
     // ### Disabled for now, since it causes problems with characters
@@ -712,7 +726,7 @@ void TEWidget::drawAttrStr(QPainter &paint, QRect rect,
     {
       // The meaning of y differs between different versions of QPainter::drawText!!
       int y = rect.y()+a; // baseline
- 
+
       if ( shadow ) {
         paint.setPen( Qt::black );
         paint.drawText(x+1,y+1, str, -1, bidiEnabled ? QPainter::Auto : QPainter::LTR );
@@ -769,7 +783,10 @@ void TEWidget::setCursorPos(const int curx, const int cury)
     int xpos, ypos;
     ypos = bY + tLy + font_h*(cury-1) + font_a;
     xpos = bX + tLx + font_w*curx;
-    setMicroFocusHint(xpos, ypos, 0, font_h);
+    /* The hasFocus() check is to avoid crashes in QXIMInputContext on some systems.
+    See http://lists.kde.org/?l=kde-core-devel&m=115770546313922&w=2 . */
+    if (hasFocus())
+      setMicroFocusHint(xpos, ypos, 0, font_h);
     // fprintf(stderr, "x/y = %d/%d\txpos/ypos = %d/%d\n", curx, cury, xpos, ypos);
     m_cursorLine = cury;
     m_cursorCol = curx;
@@ -880,7 +897,7 @@ void TEWidget::setImage(const ca* const newimg, int lines, int columns)
           if ( ( m_imSelStart < m_imSelEnd ) )
             m_isIMSel = true;
 	}
-        
+
         bool save_fixed_font = fixed_font;
         if (lineDraw)
            fixed_font = false;
@@ -1957,7 +1974,7 @@ void TEWidget::imEndEvent( QIMEvent *e )
                              contentsRect().width(), contentsRect().height() );
   m_imStart = 0;
   m_imPreeditLength = 0;
- 
+
   m_isIMEdit = m_isIMSel = false;
   repaint( repaintRect, true );
 }
@@ -2015,16 +2032,32 @@ void TEWidget::setBellMode(int mode)
 
 void TEWidget::Bell(bool visibleSession, QString message)
 {
+  if (bellTimer.isActive())
+          return;
+
+  //minimum delay in milliseconds between each bell event
+  //for the 3 different types of bells.
+  const int BELLSYSTEM_DELAY = 100;
+  const int BELLNOTIFY_DELAY = 500; //longer to avoid horrible noise with several audible system
+                                    //notifications in close succession
+  const int BELLVISUAL_DELAY = 500; //longer to avoid ugly flickering with several flashes in close
+                                    //succession
+
   if (m_bellMode==BELLNONE) return;
 
   if (m_bellMode==BELLSYSTEM) {
+    bellTimer.start(BELLSYSTEM_DELAY,true);
     KNotifyClient::beep();
   } else if (m_bellMode==BELLNOTIFY) {
+    bellTimer.start(BELLNOTIFY_DELAY,true);
+
     if (visibleSession)
       KNotifyClient::event(winId(), "BellVisible", message);
     else
       KNotifyClient::event(winId(), "BellInvisible", message);
   } else if (m_bellMode==BELLVISUAL) {
+    bellTimer.start(BELLVISUAL_DELAY,true);
+    
     swapColorTable();
     QTimer::singleShot(200,this,SLOT(swapColorTable()));
   }
