@@ -1,15 +1,23 @@
+/*
+    This file is part of Konsole, an X terminal.
+    Copyright (C) 1996 by Matthias Ettrich <ettrich@kde.org>
+    Copyright (C) 1997,1998 by Lars Doelle <lars.doelle@on-line.de>
 
-/* -------------------------------------------------------------------------- */
-/*                                                                            */
-/*                         Terminal Emulation Decoder                         */
-/*                                                                            */
-/* -------------------------------------------------------------------------- */
-/*                                                                            */
-/* Copyright (c) 1997,1998 by Lars Doelle <lars.doelle@on-line.de>            */
-/*                                                                            */
-/* This file is part of Konsole - an X terminal for KDE                       */
-/*                                                                            */
-/* -------------------------------------------------------------------------- */
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+    02110-1301  USA.
+*/
 
 /*! \class TEmulation
 
@@ -325,7 +333,7 @@ void TEmulation::onRcvBlock(const char *s, int len)
     }
 
     // Otherwise, bulk decode until the next control code
-    for(l = i; l < len; ++l)
+    for(l = i; l < len-1; ++l)
       if ((unsigned char) s[l] < 32)
          break;
 
@@ -507,6 +515,8 @@ void TEmulation::setListenToKeyPress(bool l)
 
 void TEmulation::onImageSizeChange(int lines, int columns)
 {
+  assert( lines > 0 && columns > 0 );
+
    //kdDebug(1211)<<"TEmulation::onImageSizeChange()"<<endl;
   screen[0]->resizeImage(lines,columns);
   screen[1]->resizeImage(lines,columns);
