@@ -359,6 +359,16 @@ void KSMServer::cancelShutdown( KSMClient* c )
     state = Idle;
 }
 
+void KSMServer::startProtection()
+{
+    protectionTimer.start( 10000, true );
+}
+
+void KSMServer::endProtection()
+{
+    protectionTimer.stop();
+}
+
 /*
    Internal protection slot, invoked when clients do not react during
   shutdown.
@@ -445,7 +455,7 @@ void KSMServer::startKilling()
     for ( KSMClient* c = clients.first(); c; c = clients.next() ) {
         if( isWM( c )) {
             kdDebug( 1218 ) << "Killing WM: " << c->program() << "(" << c->clientId() << ")" << endl;
-            QTimer::singleShot( 1000, this, SLOT( timeoutWMQuit()));
+            QTimer::singleShot( 2000, this, SLOT( timeoutWMQuit()));
             SmsDie( c->connection());
             return;
         }
@@ -467,7 +477,7 @@ void KSMServer::performStandardKilling()
     kdDebug( 1218 ) << " We killed all clients. We have now clients.count()=" <<
        clients.count() << endl;
     completeKilling();
-    QTimer::singleShot( 4000, this, SLOT( timeoutQuit() ) );
+    QTimer::singleShot( 10000, this, SLOT( timeoutQuit() ) );
 }
 
 void KSMServer::completeKilling()
