@@ -44,7 +44,7 @@ SignalPlotter::SignalPlotter( QWidget *parent, const char *name )
   mBeamData.setAutoDelete( false );
 
   setBackgroundMode( NoBackground );
-
+  mShowThinFrame = true;
   mSamples = 0;
   mMinValue = mMaxValue = 0.0;
   mUseAutoRange = true;
@@ -126,6 +126,25 @@ void SignalPlotter::addSample( const QValueList<double>& sampleBuf )
   update();
 }
 
+void SignalPlotter::reorderBeams( const QValueList<int>& newOrder )
+{
+  if(newOrder.count() != mBeamData.count()) {
+    kdDebug() << "Serious problem in move sample" << endl;
+    return;
+  }
+  QPtrList<double> newBeamData;
+  QValueList<QColor> newBeamColor;
+
+  for(uint i = 0; i < newOrder.count(); i++) {
+    int newIndex = newOrder[i];
+    newBeamData.append(mBeamData.at(newIndex));
+    newBeamColor.append(*mBeamColor.at(newIndex));
+  }
+  mBeamData = newBeamData;
+  mBeamColor = newBeamColor;
+
+}
+
 void SignalPlotter::changeRange( int beam, double min, double max )
 {
   // Only the first beam affects range calculation.
@@ -386,10 +405,14 @@ void SignalPlotter::paintEvent( QPaintEvent* )
   /* Draw white line along the bottom and the right side of the
    * widget to create a 3D like look. */
   p.setPen( QColor( colorGroup().light() ) );
-  p.drawLine( 0, h - 1, w - 1, h - 1 );
-  p.drawLine( w - 1, 0, w - 1, h - 1 );
+  if(mShowThinFrame) {
+    p.drawLine( 0, h - 1, w - 1, h - 1 );
+    p.drawLine( w - 1, 0, w - 1, h - 1 );
 
-  p.setClipRect( 1, 1, w - 2, h - 2 );
+    h--;
+    w--;
+    p.setClipRect( 0, 0, w, h );
+  }
   double range = mMaxValue - mMinValue;
 
   /* If the range is too small we will force it to 1.0 since it
@@ -410,21 +433,21 @@ void SignalPlotter::paintEvent( QPaintEvent* )
         range = 1.0;
     }
     // Massage the range so that the grid shows some nice values.
-    double step = range / mHorizontalLinesCount;
+    double step = range / (mHorizontalLinesCount+1);
     double dim = pow( 10, floor( log10( step ) ) ) / 2;
-    range = dim * ceil( step / dim ) * mHorizontalLinesCount;
+    range = dim * ceil( step / dim ) * (mHorizontalLinesCount+1);
   }
   double maxValue = minValue + range;
 
   int top = 0;
-  if ( mShowTopBar && h > ( mFontSize + 2 + mHorizontalLinesCount * 10 ) ) {
+  if ( mShowTopBar && h > ( mFontSize/*top bar size*/ + 2/*padding*/ +5/*smallest reasonable size for a graph*/ ) ) {
     /* Draw horizontal bar with current sensor values at top of display. */
     p.setPen( mHorizontalLinesColor );
     int x0 = w / 2;
     p.setFont( QFont( p.font().family(), mFontSize ) );
     top = p.fontMetrics().height();
     h -= top;
-    int h0 = top - 2;
+    int h0 = top - 2; // h0 is our new top.  It's at least 5 pixels high
     p.drawText(0, 0, x0, top - 2, Qt::AlignCenter, mTitle );
 
     p.drawLine( x0 - 1, 1, x0 - 1, h0 );
@@ -588,26 +611,35 @@ void SignalPlotter::paintEvent( QPaintEvent* )
     delete[] prevVals;
   }
 
-  /* Draw horizontal lines and values. Lines are drawn when the
-   * height is greater than 10 times hCount + 1, values are shown
-   * when width is greater than 60 */
-  if ( mShowHorizontalLines && h > ( 10 * ( mHorizontalLinesCount + 1 ) ) ) {
+  /* Draw horizontal lines and values. Lines are always drawn.
+   * Values are only draw when width is greater than 60 */
+  if ( mShowHorizontalLines ) {
     p.setPen( mHorizontalLinesColor );
     p.setFont( QFont( p.font().family(), mFontSize ) );
     QString val;
-    for ( uint y = 1; y < mHorizontalLinesCount; y++ ) {
-      p.drawLine( 0, top + y * ( h / mHorizontalLinesCount ), w - 2,
-                  top + y * ( h / mHorizontalLinesCount ) );
+
+    /* top = 0 or  font.height    depending on whether there's a topbar or not
+     * h = graphing area.height   - i.e. the actual space we have to draw inside
+     *
+     * Note we are drawing from 0,0 as the top left corner.  So we have to add on top to get to the top of where we are drawing
+     * so top+h is the height of the widget
+     */ 
+    for ( uint y = 1; y <= mHorizontalLinesCount; y++ ) {
+
+      int y_coord =  top + (y * h) / (mHorizontalLinesCount+1);  //Make sure it's y*h first to avoid rounding bugs
+      p.drawLine( 0, y_coord, w - 2, y_coord );
+
       if ( mShowLabels && h > ( mFontSize + 1 ) * ( mHorizontalLinesCount + 1 )
            && w > 60 ) {
-        val = QString( "%1" ).arg( maxValue - y * ( range / mHorizontalLinesCount ) );
-        p.drawText( 6, top + y * ( h / mHorizontalLinesCount ) - 1, val );
+        val = QString::number(maxValue - (y * range) / (mHorizontalLinesCount+1 ) );
+        p.drawText( 6, y_coord  - 1, val );  //draw the text one pixel raised above the line
       }
     }
 
+    //Draw the bottom most (minimum) number as well
     if ( mShowLabels && h > ( mFontSize + 1 ) * ( mHorizontalLinesCount + 1 )
          && w > 60 ) {
-      val = QString( "%1" ).arg( minValue );
+      val = QString::number( minValue );
       p.drawText( 6, top + h - 2, val );
     }
   }
