@@ -101,6 +101,9 @@ class SignalPlotter : public QWidget
 
     void setBackgroundColor( const QColor &color );
     QColor backgroundColor() const;
+    void reorderBeams( const QValueList<int>& newOrder );
+
+    void setThinFrame(bool set) { mShowThinFrame = set; }
 
   protected:
     void updateDataBuffers();
@@ -112,6 +115,7 @@ class SignalPlotter : public QWidget
     double mMinValue;
     double mMaxValue;
     bool mUseAutoRange;
+    bool mShowThinFrame;
 
     uint mGraphStyle;
 
