@@ -67,6 +67,7 @@ class Client : public QObject, public KDecorationDefines
         const Group* group() const;
         Group* group();
         void checkGroup( Group* gr = NULL, bool force = false );
+        void changeClientLeaderGroup( Group* gr );
     // prefer isXXX() instead
         NET::WindowType windowType( bool direct = false, int supported_types = SUPPORTED_WINDOW_TYPES_MASK ) const;
         const WindowRules* rules() const;
@@ -336,7 +337,6 @@ class Client : public QObject, public KDecorationDefines
         void clientMessageEvent( XClientMessageEvent* e );
         void enterNotifyEvent( XCrossingEvent* e );
         void leaveNotifyEvent( XCrossingEvent* e );
-        void visibilityNotifyEvent( XVisibilityEvent* e );
         void focusInEvent( XFocusInEvent* e );
         void focusOutEvent( XFocusOutEvent* e );
 
@@ -485,7 +485,6 @@ class Client : public QObject, public KDecorationDefines
         uint modal : 1; // NET::Modal
         uint noborder : 1;
         uint user_noborder : 1;
-        uint not_obscured : 1;
         uint urgency : 1; // XWMHints, UrgencyHint
         uint ignore_focus_stealing : 1; // don't apply focus stealing prevention to this client
         uint demands_attention : 1;
@@ -546,6 +545,8 @@ class Client : public QObject, public KDecorationDefines
         //int shadeOriginalHeight;
         bool isBMP_;
         QTimer* demandAttentionKNotifyTimer;
+
+        friend bool performTransiencyCheck();
     };
 
 // helper for Client::postponeGeometryUpdates() being called in pairs (true/false)
