@@ -40,6 +40,8 @@ class WindowRules
         WindowRules();
         void update( Client* );
         void discardTemporary();
+        bool contains( const Rules* rule ) const;
+        void remove( Rules* rule );
         Placement::Policy checkPlacement( Placement::Policy placement ) const;
         QRect checkGeometry( QRect rect, bool init = false ) const;
         // use 'invalidPoint' with checkPosition, unlike QSize() and QRect(), QPoint() is a valid point
@@ -49,7 +51,7 @@ class WindowRules
         QSize checkMaxSize( QSize s ) const;
         int checkOpacityActive(int s) const;
         int checkOpacityInactive(int s) const;
-        bool checkIgnorePosition( bool ignore ) const;
+        bool checkIgnoreGeometry( bool ignore ) const;
         int checkDesktop( int desktop, bool init = false ) const;
         NET::WindowType checkType( NET::WindowType type ) const;
         MaximizeMode checkMaximize( MaximizeMode mode, bool init = false ) const;
@@ -67,6 +69,8 @@ class WindowRules
         bool checkCloseable( bool closeable ) const;
         bool checkStrictGeometry( bool strict ) const;
         QString checkShortcut( QString s, bool init = false ) const;
+        bool checkDisableGlobalShortcuts( bool disable ) const;
+        bool checkIgnorePosition( bool ignore ) const; // obsolete
     private:
         MaximizeMode checkMaximizeVert( MaximizeMode mode, bool init ) const;
         MaximizeMode checkMaximizeHoriz( MaximizeMode mode, bool init ) const;
@@ -84,6 +88,7 @@ class Rules
         void write( KConfig& ) const;
         bool isEmpty() const;
 #ifndef KCMRULES
+        void discardUsed( bool withdrawn );
         bool match( const Client* c ) const;
         bool update( Client* );
         bool isTemporary() const;
@@ -97,7 +102,7 @@ class Rules
         bool applyMaxSize( QSize& s ) const;
         bool applyOpacityActive(int& s) const;
         bool applyOpacityInactive(int& s) const;
-        bool applyIgnorePosition( bool& ignore ) const;
+        bool applyIgnoreGeometry( bool& ignore ) const;
         bool applyDesktop( int& desktop, bool init ) const;
         bool applyType( NET::WindowType& type ) const;
         bool applyMaximizeVert( MaximizeMode& mode, bool init ) const;
@@ -116,6 +121,8 @@ class Rules
         bool applyCloseable( bool& closeable ) const;
         bool applyStrictGeometry( bool& strict ) const;
         bool applyShortcut( QString& shortcut, bool init ) const;
+        bool applyDisableGlobalShortcuts( bool& disable ) const;
+        bool applyIgnorePosition( bool& ignore ) const; // obsolete
     private:
 #endif
         bool matchType( NET::WindowType match_type ) const;
@@ -130,7 +137,9 @@ class Rules
             DontAffect, // use the default value
             Force,      // force the given value
             Apply,      // apply only after initial mapping
-            Remember   // like apply, and remember the value when the window is withdrawn
+            Remember,   // like apply, and remember the value when the window is withdrawn
+            ApplyNow,   // apply immediatelly, then forget the setting
+            ForceTemporarily // apply and force until the window is withdrawn
             };
         enum SetRule
             {
@@ -227,6 +236,8 @@ class Rules
         ForceRule strictgeometryrule;
         QString shortcut;
         SetRule shortcutrule;
+        bool disableglobalshortcuts;
+        ForceRule disableglobalshortcutsrule;
         friend kdbgstream& operator<<( kdbgstream& stream, const Rules* );
     };
 
@@ -236,7 +247,8 @@ bool Rules::checkSetRule( SetRule rule, bool init )
     {
     if( rule > ( SetRule )DontAffect) // Unused or DontAffect
         {
-        if( rule == ( SetRule )Force || init )
+        if( rule == ( SetRule )Force || rule == ( SetRule ) ApplyNow
+            || rule == ( SetRule ) ForceTemporarily || init )
             return true;
         }
     return false;
@@ -245,7 +257,7 @@ bool Rules::checkSetRule( SetRule rule, bool init )
 inline
 bool Rules::checkForceRule( ForceRule rule )
     {
-    return rule == ( ForceRule )Force;
+    return rule == ( ForceRule )Force || rule == ( ForceRule ) ForceTemporarily;
     }
 
 inline
@@ -270,6 +282,21 @@ inline
 WindowRules::WindowRules()
     {
     }
+
+inline
+bool WindowRules::contains( const Rules* rule ) const
+    {
+    return qFind( rules.begin(), rules.end(), rule ) != rules.end();
+    }
+    
+inline
+void WindowRules::remove( Rules* rule )
+    {
+    QValueVector< Rules* >::Iterator pos = qFind( rules.begin(), rules.end(), rule );
+    if( pos != rules.end())
+        rules.erase( pos );
+    }
+
 #endif
 
 #ifdef NDEBUG
