@@ -44,7 +44,7 @@ namespace KWinInternal
 extern QPixmap* kwin_get_menu_pix_hack();
 
 TabBox::TabBox( Workspace *ws, const char *name )
-    : QFrame( 0, name, Qt::WNoAutoErase ), client(0), wspace(ws)
+    : QFrame( 0, name, Qt::WNoAutoErase ), current_client( NULL ), wspace(ws)
     {
     setFrameStyle(QFrame::StyledPanel | QFrame::Plain);
     setLineWidth(2);
@@ -58,10 +58,25 @@ TabBox::TabBox( Workspace *ws, const char *name )
     reconfigure();
     reset();
     connect(&delayedShowTimer, SIGNAL(timeout()), this, SLOT(show()));
+    
+    XSetWindowAttributes attr;
+    attr.override_redirect = 1;
+    outline_left = XCreateWindow( qt_xdisplay(), qt_xrootwin(), 0, 0, 1, 1, 0,
+        CopyFromParent, CopyFromParent, CopyFromParent, CWOverrideRedirect, &attr );
+    outline_right = XCreateWindow( qt_xdisplay(), qt_xrootwin(), 0, 0, 1, 1, 0,
+        CopyFromParent, CopyFromParent, CopyFromParent, CWOverrideRedirect, &attr );
+    outline_top = XCreateWindow( qt_xdisplay(), qt_xrootwin(), 0, 0, 1, 1, 0,
+        CopyFromParent, CopyFromParent, CopyFromParent, CWOverrideRedirect, &attr );
+    outline_bottom = XCreateWindow( qt_xdisplay(), qt_xrootwin(), 0, 0, 1, 1, 0,
+        CopyFromParent, CopyFromParent, CopyFromParent, CWOverrideRedirect, &attr );
     }
 
 TabBox::~TabBox()
     {
+    XDestroyWindow( qt_xdisplay(), outline_left );
+    XDestroyWindow( qt_xdisplay(), outline_right );
+    XDestroyWindow( qt_xdisplay(), outline_top );
+    XDestroyWindow( qt_xdisplay(), outline_bottom );
     }
 
 
@@ -150,10 +165,10 @@ void TabBox::reset()
 
     if ( mode() == WindowsMode )
         {
-        client = workspace()->activeClient();
+        setCurrentClient( workspace()->activeClient());
 
         // get all clients to show
-        createClientList(clients, options_traverse_all ? -1 : workspace()->currentDesktop(), client, true);
+        createClientList(clients, options_traverse_all ? -1 : workspace()->currentDesktop(), current_client, true);
 
         // calculate maximum caption width
         cw = fontMetrics().width(no_tasks)+20;
@@ -230,7 +245,8 @@ void TabBox::nextPrev( bool next)
     {
     if ( mode() == WindowsMode )
         {
-        Client* firstClient = 0;
+        Client* firstClient = NULL;
+        Client* client = current_client;
         do
             {
             if ( next )
@@ -250,6 +266,7 @@ void TabBox::nextPrev( bool next)
                 break;
                 }
             } while ( client && !clients.contains( client ));
+        setCurrentClient( client );
         }
     else if( mode() == DesktopMode )
         {
@@ -287,9 +304,18 @@ Client* TabBox::currentClient()
     {
     if ( mode() != WindowsMode )
         return 0;
-    if (!workspace()->hasClient( client ))
+    if (!workspace()->hasClient( current_client ))
         return 0;
-    return client;
+    return current_client;
+    }
+
+void TabBox::setCurrentClient( Client* c )
+    {
+    if( current_client != c )
+        {
+        current_client = c;
+        updateOutline();
+        }
     }
 
 /*!
@@ -311,6 +337,11 @@ int TabBox::currentDesktop()
  */
 void TabBox::showEvent( QShowEvent* )
     {
+    updateOutline();
+    XRaiseWindow( qt_xdisplay(), outline_left );
+    XRaiseWindow( qt_xdisplay(), outline_right );
+    XRaiseWindow( qt_xdisplay(), outline_top );
+    XRaiseWindow( qt_xdisplay(), outline_bottom );
     raise();
     }
 
@@ -320,6 +351,10 @@ void TabBox::showEvent( QShowEvent* )
  */
 void TabBox::hideEvent( QHideEvent* )
     {
+    XUnmapWindow( qt_xdisplay(), outline_left );
+    XUnmapWindow( qt_xdisplay(), outline_right );
+    XUnmapWindow( qt_xdisplay(), outline_top );
+    XUnmapWindow( qt_xdisplay(), outline_bottom );
     }
 
 /*!
@@ -358,7 +393,7 @@ void TabBox::drawContents( QPainter * )
               if ( workspace()->hasClient( *it ) )  // safety
                   {
                   // draw highlight background
-                  if ( (*it) == currentClient() )
+                  if ( (*it) == current_client )
                     p.fillRect(x, y, r.width(), lineHeight, colorGroup().highlight());
 
                   // draw icon
@@ -395,7 +430,7 @@ void TabBox::drawContents( QPainter * )
                   s = KStringHandler::cPixelSqueeze(s, fontMetrics(), r.width() - 5 - iconWidth - 8);
 
                   // draw text
-                  if ( (*it) == currentClient() )
+                  if ( (*it) == current_client )
                     p.setPen(colorGroup().highlightedText());
                   else if( (*it)->isMinimized())
                     {
@@ -511,6 +546,91 @@ void TabBox::drawContents( QPainter * )
     bitBlt(this, r.x(), r.y(), &pix);
     }
 
+void TabBox::updateOutline()
+    {
+    Client* c = currentClient();
+    if( c == NULL || this->isHidden() || !c->isShown( true ) || !c->isOnCurrentDesktop())
+        {
+        XUnmapWindow( qt_xdisplay(), outline_left );
+        XUnmapWindow( qt_xdisplay(), outline_right );
+        XUnmapWindow( qt_xdisplay(), outline_top );
+        XUnmapWindow( qt_xdisplay(), outline_bottom );
+        return;
+        }
+    // left/right parts are between top/bottom, they don't reach as far as the corners
+    XMoveResizeWindow( qt_xdisplay(), outline_left, c->x(), c->y() + 5, 5, c->height() - 10 );
+    XMoveResizeWindow( qt_xdisplay(), outline_right, c->x() + c->width() - 5, c->y() + 5, 5, c->height() - 10 );
+    XMoveResizeWindow( qt_xdisplay(), outline_top, c->x(), c->y(), c->width(), 5 );
+    XMoveResizeWindow( qt_xdisplay(), outline_bottom, c->x(), c->y() + c->height() - 5, c->width(), 5 );
+    {
+    QPixmap pix( 5, c->height() - 10 );
+    QPainter p( &pix );
+    p.setPen( white );
+    p.drawLine( 0, 0, 0, pix.height() - 1 );
+    p.drawLine( 4, 0, 4, pix.height() - 1 );
+    p.setPen( gray );
+    p.drawLine( 1, 0, 1, pix.height() - 1 );
+    p.drawLine( 3, 0, 3, pix.height() - 1 );
+    p.setPen( black );
+    p.drawLine( 2, 0, 2, pix.height() - 1 );
+    p.end();
+    XSetWindowBackgroundPixmap( qt_xdisplay(), outline_left, pix.handle());
+    XSetWindowBackgroundPixmap( qt_xdisplay(), outline_right, pix.handle());
+    }
+    {
+    QPixmap pix( c->width(), 5 );
+    QPainter p( &pix );
+    p.setPen( white );
+    p.drawLine( 0, 0, pix.width() - 1 - 0, 0 );
+    p.drawLine( 4, 4, pix.width() - 1 - 4, 4 );
+    p.drawLine( 0, 0, 0, 4 );
+    p.drawLine( pix.width() - 1 - 0, 0, pix.width() - 1 - 0, 4 );
+    p.setPen( gray );
+    p.drawLine( 1, 1, pix.width() - 1 - 1, 1 );
+    p.drawLine( 3, 3, pix.width() - 1 - 3, 3 );
+    p.drawLine( 1, 1, 1, 4 );
+    p.drawLine( 3, 3, 3, 4 );
+    p.drawLine( pix.width() - 1 - 1, 1, pix.width() - 1 - 1, 4 );
+    p.drawLine( pix.width() - 1 - 3, 3, pix.width() - 1 - 3, 4 );
+    p.setPen( black );
+    p.drawLine( 2, 2, pix.width() - 1 - 2, 2 );
+    p.drawLine( 2, 2, 2, 4 );
+    p.drawLine( pix.width() - 1 - 2, 2, pix.width() - 1 - 2, 4 );
+    p.end();
+    XSetWindowBackgroundPixmap( qt_xdisplay(), outline_top, pix.handle());
+    }
+    {
+    QPixmap pix( c->width(), 5 );
+    QPainter p( &pix );
+    p.setPen( white );
+    p.drawLine( 4, 0, pix.width() - 1 - 4, 0 );
+    p.drawLine( 0, 4, pix.width() - 1 - 0, 4 );
+    p.drawLine( 0, 4, 0, 0 );
+    p.drawLine( pix.width() - 1 - 0, 4, pix.width() - 1 - 0, 0 );
+    p.setPen( gray );
+    p.drawLine( 3, 1, pix.width() - 1 - 3, 1 );
+    p.drawLine( 1, 3, pix.width() - 1 - 1, 3 );
+    p.drawLine( 3, 1, 3, 0 );
+    p.drawLine( 1, 3, 1, 0 );
+    p.drawLine( pix.width() - 1 - 3, 1, pix.width() - 1 - 3, 0 );
+    p.drawLine( pix.width() - 1 - 1, 3, pix.width() - 1 - 1, 0 );
+    p.setPen( black );
+    p.drawLine( 2, 2, pix.width() - 1 - 2, 2 );
+    p.drawLine( 2, 0, 2, 2 );
+    p.drawLine( pix.width() - 1 - 2, 0, pix.width() - 1 - 2, 2 );
+    p.end();
+    XSetWindowBackgroundPixmap( qt_xdisplay(), outline_bottom, pix.handle());
+    }
+    XClearWindow( qt_xdisplay(), outline_left );
+    XClearWindow( qt_xdisplay(), outline_right );
+    XClearWindow( qt_xdisplay(), outline_top );
+    XClearWindow( qt_xdisplay(), outline_bottom );
+    XMapWindow( qt_xdisplay(), outline_left );
+    XMapWindow( qt_xdisplay(), outline_right );
+    XMapWindow( qt_xdisplay(), outline_top );
+    XMapWindow( qt_xdisplay(), outline_bottom );
+    }
+
 void TabBox::hide()
     {
     delayedShowTimer.stop();
@@ -587,7 +707,7 @@ void TabBox::handleMouseEvent( XEvent* e )
             {
             if( workspace()->hasClient( *it ) && (num == 0) ) // safety
                 {
-                client = *it;
+                setCurrentClient( *it );
                 break;
                 }
             num--;
@@ -881,9 +1001,9 @@ bool Workspace::startKDEWalkThroughWindows()
     if( !establishTabBoxGrab())
         return false;
     tab_grab        = TRUE;
-    keys->setEnabled( false );
-    disable_shortcuts_keys->setEnabled( false );
-    client_keys->setEnabled( false );
+    keys->suspend( true );
+    disable_shortcuts_keys->suspend( true );
+    client_keys->suspend( true );
     tab_box->setMode( TabBox::WindowsMode );
     tab_box->reset();
     return TRUE;
@@ -894,9 +1014,9 @@ bool Workspace::startWalkThroughDesktops( int mode )
     if( !establishTabBoxGrab())
         return false;
     control_grab = TRUE;
-    keys->setEnabled( false );
-    disable_shortcuts_keys->setEnabled( false );
-    client_keys->setEnabled( false );
+    keys->suspend( true );
+    disable_shortcuts_keys->suspend( true );
+    client_keys->suspend( true );
     tab_box->setMode( (TabBox::Mode) mode );
     tab_box->reset();
     return TRUE;
@@ -1064,9 +1184,9 @@ void Workspace::closeTabBox()
     {
     removeTabBoxGrab();
     tab_box->hide();
-    keys->setEnabled( true );
-    disable_shortcuts_keys->setEnabled( true );
-    client_keys->setEnabled( true );
+    keys->suspend( false );
+    disable_shortcuts_keys->suspend( false );
+    client_keys->suspend( false );
     tab_grab = FALSE;
     control_grab = FALSE;
     }
@@ -1113,9 +1233,9 @@ void Workspace::tabBoxKeyRelease( const XKeyEvent& ev )
         {
         removeTabBoxGrab();
         tab_box->hide();
-        keys->setEnabled( true );
-        disable_shortcuts_keys->setEnabled( true );
-        client_keys->setEnabled( true );
+        keys->suspend( false );
+        disable_shortcuts_keys->suspend( false );
+        client_keys->suspend( false );
         tab_grab = false;
         if( Client* c = tab_box->currentClient())
             {
@@ -1128,9 +1248,9 @@ void Workspace::tabBoxKeyRelease( const XKeyEvent& ev )
         {
         removeTabBoxGrab();
         tab_box->hide();
-        keys->setEnabled( true );
-        disable_shortcuts_keys->setEnabled( true );
-        client_keys->setEnabled( true );
+        keys->suspend( false );
+        disable_shortcuts_keys->suspend( false );
+        client_keys->suspend( false );
         control_grab = False;
         if ( tab_box->currentDesktop() != -1 )
             {
