@@ -33,6 +33,7 @@ class TabBox : public QFrame
         ~TabBox();
 
         Client* currentClient();
+        void setCurrentClient( Client* c );
         int currentDesktop();
 
     // DesktopMode and WindowsMode are based on the order in which the desktop
@@ -62,9 +63,10 @@ class TabBox : public QFrame
 
     private:
         void createClientList(ClientList &list, int desktop /*-1 = all*/, Client *start, bool chain);
+        void updateOutline();
 
     private:
-        Client* client;
+        Client* current_client;
         Mode m;
         Workspace* wspace;
         ClientList clients;
@@ -74,6 +76,7 @@ class TabBox : public QFrame
         QTimer delayedShowTimer;
         QString no_tasks;
         bool options_traverse_all;
+        Window outline_left, outline_right, outline_top, outline_bottom;
     };
 
 
